package com.gfeng.daydaycook.activity

import android.os.Bundle
import android.view.View
import android.widget.AbsListView
import android.widget.AdapterView
import com.alibaba.fastjson.JSONObject
import com.alibaba.fastjson.TypeReference
import com.gfeng.daydaycook.R
import com.gfeng.daydaycook.adapter.CaltalogAdapter
import com.gfeng.daydaycook.adapter.CaltalogTypeAdapter
import com.gfeng.daydaycook.comm.Comm
import com.gfeng.daydaycook.comm.Global
import com.gfeng.daydaycook.logic.NotifyMgr
import com.gfeng.daydaycook.model.CatalogShowModel
import com.gfeng.daydaycook.model.HotCatalogModel
import com.gfeng.daydaycook.model.ResponseModel
import com.gfeng.daydaycook.util.LogUtils
import kotlinx.android.synthetic.main.activity_hot_caltalog.*
import kotlinx.android.synthetic.main.top_layout.*
import java.util.*

/**
 * 食谱分类
 */
open class HotCaltalogActivity : BaseActivity(), AdapterView.OnItemClickListener, AbsListView.OnScrollListener {

    private var caltalogAdapter: CaltalogAdapter? = null

    private var caltalogTypeAdapter: CaltalogTypeAdapter? = null

    private var firstVisibleItem = 0


    override fun aidHandleMessage(what: Int, obj: Any?) {

        try {
            when (what) {
                Comm.reloadData -> {
                    no_internet.visibility = View.GONE
                    initData()
                }
                ON_SUCCESS -> {
                    hideProgressDialog()
                    if (obj == null) {
                        return
                    }
                    val responseModel = obj as ResponseModel?

                    if (responseModel!!.code != Comm.CODE_200) {
                        NotifyMgr.showShortToast(responseModel.data.toString())
                        return
                    }

                    when (responseModel.type) {
                        hotCataloglist_network_refresh_type -> {

                            val hotCatalogModel = responseModel.data as HotCatalogModel

                            val list = ArrayList<CatalogShowModel>()
                            list.add(hotCatalogModel.catagoryHotVo)
                            list.addAll(hotCatalogModel.catagoryRecommendVos)

                            caltalogAdapter?.mList = list
                            caltalogAdapter?.notifyDataSetChanged()

                            caltalogTypeAdapter?.selectCatalogShowModel = list[0]
                            caltalogTypeAdapter?.mList = list
                            caltalogTypeAdapter?.notifyDataSetChanged()

                        }
                    }
                }
                ON_FAILED -> no_internet.visibility = View.VISIBLE
            }

        } catch (e: Throwable) {
            LogUtils.e(TAG, e)
        }

    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_hot_caltalog)
        Global.mAppMgr.setActivityDataRefreshListener(this, Comm.hot_caltalog_activity_refresh_type)

        init()
    }

    private fun init() {
        mTitle.setText(R.string.hot_catalog_title)
        backButton.setOnClickListener { finish() }

        initUi()
        initData()
        initListener()

    }


    internal fun initUi() {

        val list = ArrayList<CatalogShowModel>()
        caltalogAdapter = CaltalogAdapter(this, list)
        rightListView.adapter = caltalogAdapter

        caltalogTypeAdapter = CaltalogTypeAdapter(this, list)
        leftListView.adapter = caltalogTypeAdapter

    }

    internal fun initData() {
        showProgressDialog()
        httpSend(object : TypeReference<HotCatalogModel>() {}.type, Global.mcnHost, Comm.getMcnURL(Comm.RECIPE_CLASSIFICATION), JSONObject(), hotCataloglist_network_refresh_type)
    }


    private fun initListener() {
        leftListView.onItemClickListener = this
        rightListView.setOnScrollListener(this)

    }

    override fun onItemClick(parent: AdapterView<*>, view: View, position: Int, id: Long) {
        when (parent.id) {
            R.id.leftListView -> {
                val leftModel = caltalogTypeAdapter?.mList?.get(position)
                caltalogTypeAdapter?.selectCatalogShowModel = leftModel
                caltalogTypeAdapter?.notifyDataSetInvalidated()

                rightListView.setSelection(caltalogAdapter?.mList?.indexOf(leftModel) ?: 0)
            }
        }
    }

    override fun onScrollStateChanged(view: AbsListView, scrollState: Int) {
        when (scrollState) {
            AbsListView.OnScrollListener.SCROLL_STATE_IDLE -> {
                val model = caltalogAdapter?.mList?.get(firstVisibleItem)
                caltalogTypeAdapter?.selectCatalogShowModel = model
                caltalogTypeAdapter?.notifyDataSetInvalidated()
                leftListView.setSelection(caltalogTypeAdapter?.mList?.indexOf(model) ?: 0)
            }
            AbsListView.OnScrollListener.SCROLL_STATE_TOUCH_SCROLL -> {
            }
            AbsListView.OnScrollListener.SCROLL_STATE_FLING -> LogUtils.info("开始滚动...")
        }
    }

    override fun onScroll(view: AbsListView, firstVisibleItem: Int, visibleItemCount: Int, totalItemCount: Int) {
        this.firstVisibleItem = firstVisibleItem
    }


    override fun onDestroy() {
        super.onDestroy()
        Global.mAppMgr.setActivityDataRefreshListener(null, Comm.hot_caltalog_activity_refresh_type)
    }

    companion object {

        private val TAG = HotCaltalogActivity::class.java.simpleName
        private const val hotCataloglist_network_refresh_type = 100
    }
}
