package com.gfeng.daydaycook.activity


import android.app.Activity
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.os.Environment
import android.provider.MediaStore
import android.view.LayoutInflater
import android.view.Menu
import android.view.View
import android.view.ViewGroup
import android.widget.*
import android.widget.AdapterView.OnItemClickListener
import com.gfeng.daydaycook.R
import com.gfeng.daydaycook.adapter.ImageChooseViewAdapter
import com.gfeng.daydaycook.comm.Comm
import com.gfeng.daydaycook.model.AlbumItem
import com.gfeng.daydaycook.util.DensityUtil
import com.jiuli.library.ui.WebImageView
import kotlinx.android.synthetic.main.activity_image_choice.*
import java.io.File
import java.util.*

class ImageChoiceActivity : BaseActivity(), OnItemClickListener, ImageChooseViewAdapter.OnChechedListener {

    var adapter: ImageChooseViewAdapter? = null
    var allFileList: MutableList<AlbumItem> = ArrayList()
    var fileList: MutableList<AlbumItem> = ArrayList()
    var folderList: MutableList<File> = ArrayList()
    var thumbnailMap: MutableMap<String, String> = TreeMap()
    var folderMap: MutableMap<String, File> = TreeMap()
    var fileCountMap: MutableMap<String, Int> = HashMap()
    private var button: Button? = null
    private var maxCount: Int = 0
    private var folderAdapter: FolderAdapter? = null
    private var selectedFolderIndex = 0
    internal var currentFolder: File? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_image_choice)
        init()
    }

    fun init() {

        folderSelectPanel.setOnClickListener(this)
        folderSelect.setOnClickListener(this)
        gridView.onItemClickListener = this
        maxCount = intent.getIntExtra(KEY_MAX_COUNT, 1)
        adapter = ImageChooseViewAdapter(this, fileList, maxCount)
        gridView.adapter = adapter
        adapter?.notifyDataSetChanged()
        adapter?.setOnChechedListener(this)

        listView.onItemClickListener = this
        folderAdapter = FolderAdapter()
        listView.adapter = folderAdapter

        searchImageList()
    }

    override fun onClick(v: View) {
        when (v.id) {
            R.id.folderSelect -> if (listView.visibility == View.GONE && folderList!!.size > 1) {
                listView.visibility = View.VISIBLE
            } else {
                listView.visibility = View.GONE
            }
            R.id.button -> {

                val intent = Intent()
                intent.putExtra("files", adapter?.selectedFiles)
                setResult(Activity.RESULT_OK, intent)
                this.finish()
            }
        }

    }

    override fun onItemClick(arg0: AdapterView<*>, arg1: View, position: Int, arg3: Long) {

        if (arg0 is GridView) {
            val item = adapter?.getItem(position)
            if (item != null) {
                val intent = Intent()
                intent.data = Uri.fromFile(item.file)
                intent.putExtra("file", item.file)
                setResult(Comm.IMAGE_ACTIVITY_CHOICE, intent)
            }
            this.finish()
            return
        }

        listView.visibility = View.GONE

        if (position == 0 && currentFolder != null) {
            onFolderChanged(null)
            currentFolder = null
            selectedFolderIndex = 0
        }

        if (position > 0 && folderList!![position] != currentFolder) {
            currentFolder = folderList!![position]
            onFolderChanged(currentFolder)
            selectedFolderIndex = position
        }

    }

    private fun searchImageList() {
        scanImages()
        adapter?.notifyDataSetChanged()
    }


    override fun onCreateOptionsMenu(menu: Menu): Boolean {

        if (maxCount > 1) {
            menuInflater.inflate(R.menu.single_button, menu)
            button = menu.findItem(R.id.menu_button).actionView.findViewById<View>(R.id.button) as Button
            button!!.setOnClickListener(this)
            button!!.text = "确定"
            button!!.isEnabled = false
        }
        return super.onCreateOptionsMenu(menu)
    }


    private fun scanImages() {
        folderList?.clear()
        if (android.os.Build.VERSION.SDK_INT > 18) {
            sendBroadcast(Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE, Uri.parse("file://" + Environment.getExternalStorageDirectory())))
        } else {
            sendBroadcast(Intent(Intent.ACTION_MEDIA_MOUNTED, Uri.parse("file://" + Environment.getExternalStorageDirectory())))
        }
        getThumbnail()
        val uri = MediaStore.Images.Media.EXTERNAL_CONTENT_URI
        val selection = MediaStore.Images.Media.MIME_TYPE + "=? or " + MediaStore.Images.Media.MIME_TYPE + "=?"
        val selectionArgs = arrayOf("image/jpeg", "image/png")
        val contentResolver = contentResolver
        val cursor = contentResolver.query(uri, null, selection, selectionArgs, MediaStore.Images.Media.DATE_ADDED + " desc")
        if (cursor != null) {
            while (cursor.moveToNext()) {
                val item = AlbumItem()
                val file = File(cursor.getString(cursor.getColumnIndex(MediaStore.Images.Media.DATA)))
                item.file = file
                item.imageId = cursor.getString(cursor.getColumnIndex("_id"))
                item.thumbnail = thumbnailMap[item.imageId]

                item.time = cursor.getLong(cursor.getColumnIndex(MediaStore.Images.Media.DATE_ADDED))
                fileList.add(item)
                val folder: String = file.parentFile.absolutePath
                if (!folderMap.containsKey(folder)) {
                    folderMap[folder] = file
                    fileCountMap[folder] = 1
                } else {
                    fileCountMap[folder] = fileCountMap[folder]!!.plus(1)
                }
                if (!folderList.contains(file.parentFile)) {
                    folderList.add(file.parentFile)
                }
            }
            cursor.close()
        }

        //Collections.sort(fileList, new TimeAscComparator());
        allFileList!!.addAll(fileList!!)
        if (fileList!!.size > 0) {
            folderAdapter?.notifyDataSetChanged()
        }
    }


    private fun getThumbnail() {
        val uri = MediaStore.Images.Thumbnails.EXTERNAL_CONTENT_URI
        val contentResolver = contentResolver
        val cursor = contentResolver.query(uri, null, null, null, null)
        if (cursor != null) {
            cursor.moveToFirst()
            if (cursor.moveToNext()) {
                val id = cursor.getString(cursor.getColumnIndex(MediaStore.Images.Thumbnails.IMAGE_ID))
                val data = cursor.getString(cursor.getColumnIndex(MediaStore.Images.Thumbnails.DATA))
                thumbnailMap[id] = data
            }
        }
        cursor?.close()
    }

    override fun onChecked(compoundbutton: CompoundButton, file: File, isChecked: Boolean) {
        if (isChecked) {

            if (adapter!!.selectedFiles.size > maxCount) {
                compoundbutton.isChecked = false
                adapter!!.selectedFiles.remove(file)
            }
        } else {
            adapter!!.selectedFiles.remove(file)
        }

        if (adapter!!.selectedFiles.size > 0) {
            val params = LinearLayout.LayoutParams(DensityUtil.dip2px(this, 65f), DensityUtil.dip2px(this, 35f))
            button!!.layoutParams = params
            button!!.isEnabled = true
            button!!.text = getString(R.string.label_album_selected_count, adapter!!.selectedFiles.size.toString() + "/" + maxCount)
        } else {
            val params = LinearLayout.LayoutParams(DensityUtil.dip2px(this, 50f), DensityUtil.dip2px(this, 35f))
            button!!.layoutParams = params
            button!!.isEnabled = false
            button!!.text = "确定"
        }

    }

    private fun onFolderChanged(file: File?) {
        fileList.clear()
        if (file == null) {
            folderSelectText.text = "所有图片"
            fileList.addAll(allFileList)
        } else {
            for (item in allFileList) {
                if (item.file.parentFile == file) {
                    fileList.add(item)
                }
            }

            folderSelectText.text = file.name
        }
        adapter?.notifyDataSetChanged()
        folderAdapter?.notifyDataSetChanged()
    }

    override fun onBackPressed() {
        if (listView.visibility == View.VISIBLE) {
            listView.visibility = View.GONE
        } else {
            allFileList.clear()
            fileList.clear()
            folderList.clear()
            thumbnailMap.clear()
            folderMap.clear()
            fileCountMap.clear()

            finish()
        }
    }

    inner class FolderAdapter : BaseAdapter() {
        override fun getCount(): Int {
            return folderList.size
        }

        override fun getItem(position: Int): File? {
            return folderList[position]
        }

        override fun getItemId(position: Int): Long {
            return 0
        }

        override fun getView(position: Int, view: View?, parent: ViewGroup): View {
            var convertView = view
            val file = getItem(position)

            if (convertView == null) {
                convertView = LayoutInflater.from(this@ImageChoiceActivity).inflate(R.layout.item_change_folder, null)
            }

            convertView!!.tag = file

            val name = convertView.findViewById<View>(R.id.name) as TextView
            val count = convertView.findViewById<View>(R.id.count) as TextView
            val image = convertView.findViewById<View>(R.id.image) as WebImageView
            if (null == file) {
                name.text = "所有图片"
                count.text = getString(R.string.label_ablbum_count, ""+allFileList.size)
                image.load("file://" + allFileList[0].file.absolutePath, null)
            } else {
                val folder = file.absolutePath
                image.load("file://" + folderMap[folder]?.absolutePath, null)
                name.text = file.name
                count.text = getString(R.string.label_ablbum_count, fileCountMap[folder] ?: "")
            }

            if (file == null && currentFolder == null || file != null && file == currentFolder) {
                convertView.findViewById<View>(R.id.mark).visibility = View.VISIBLE
            } else {
                convertView.findViewById<View>(R.id.mark).visibility = View.GONE
            }

            return convertView
        }
    }

    companion object {
        var KEY_MAX_COUNT = "KEY_MAX_COUNT"
    }
}
