package com.gfeng.daydaycook.activity

import android.Manifest
import android.annotation.SuppressLint
import android.annotation.TargetApi
import android.app.Activity
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.os.SystemClock
import android.provider.MediaStore
import android.text.TextUtils
import android.view.KeyEvent
import android.webkit.JavascriptInterface
import android.webkit.ValueCallback
import com.gfeng.daydaycook.R
import com.gfeng.daydaycook.comm.Comm
import com.gfeng.daydaycook.comm.Global
import com.gfeng.daydaycook.util.*
import com.master.permissionhelper.PermissionHelper
import com.tencent.smtt.sdk.WebChromeClient
import com.tencent.smtt.sdk.WebSettings
import com.tencent.smtt.sdk.WebView
import com.tencent.smtt.sdk.WebViewClient
import kotlinx.android.synthetic.main.activity_load_url.*
import kotlinx.android.synthetic.main.top_layout.*
import java.io.File

open class LoadUrlActivity : BaseActivity() {


    private var mUrl = ""
    internal var title = ""
    private var failUrl: String? = null
    private val errorUrl = "file:///android_asset/error.html"

    private var fileUri: File? = null
    private var imageUri: Uri? = null
    private var mUploadMessage: ValueCallback<Uri>? = null
    private var mFilePathCallback: ValueCallback<Array<Uri>>? = null

    private var webViewClient: WebViewClient = object : WebViewClient() {
        override fun shouldOverrideUrlLoading(view: WebView?, url: String): Boolean {
            LogUtils.e(TAG, "shouldOverrideUrlLoading url==$url")

            if (url.startsWith("http:") || url.startsWith("https:")) {
                view!!.loadUrl(url)
                return false
            } else {
                return true
            }
        }

        override fun onPageStarted(view: WebView?, url: String, favicon: Bitmap?) {
            LogUtils.e(TAG, "onPageStarted url==$url")
            super.onPageStarted(view, url, favicon)
            if (customProgressDialog == null || !customProgressDialog.isShowing) {
                showProgressDialog()
            }
        }

        override fun onPageFinished(view: WebView?, url: String?) {
            LogUtils.e(TAG, "onPageFinished url==" + url!!)
            super.onPageFinished(view, url)
            hideProgressDialog()
        }

        override fun onReceivedError(view: WebView?, errorCode: Int, description: String?, failingUrl: String?) {

            LogUtils.e(TAG, "onReceivedError errorCode==$errorCode  description==$description  failingUrl==$failingUrl")
            hideProgressDialog()
            if (errorCode != -10) {
                super.onReceivedError(view, errorCode, description, failingUrl)
                failUrl = failingUrl
                view!!.loadUrl(errorUrl)
            }
        }
    }

    private var webChromeClient: WebChromeClient = object : WebChromeClient() {
        override fun onProgressChanged(view: WebView?, newProgress: Int) {
            super.onProgressChanged(view, newProgress)
            if (newProgress == 100) {
                hideProgressDialog()
            }
        }


        // For Android 5.0+
        fun onShowFileChooser(webView: WebView?, filePathCallback: ValueCallback<Array<Uri>>, fileChooserParams: WebChromeClient.FileChooserParams): Boolean {
            LogUtils.e(TAG, "onShowFileChooser(ValueCallback<Uri> uploadMsg, String acceptType, String capture)")
            mFilePathCallback = filePathCallback

            takePhoto()

            return true
        }

        //For Android 4.1
        override fun openFileChooser(uploadMsg: com.tencent.smtt.sdk.ValueCallback<Uri>, acceptType: String?, capture: String?) {
            LogUtils.e(TAG, "openFileChoose(ValueCallback<Uri> uploadMsg, String acceptType, String capture)")
            mUploadMessage = uploadMsg

            //            takePhoto();
            startPhotoPicker()

        }

        // For Android 5.0+
        override fun onShowFileChooser(webView: WebView?, filePathCallback: com.tencent.smtt.sdk.ValueCallback<Array<Uri>>?, fileChooserParams: WebChromeClient.FileChooserParams?): Boolean {
            LogUtils.e(TAG, "onShowFileChooser(WebView webView, ValueCallback<Uri[]> filePathCallback, FileChooserParams fileChooserParams)")
            mFilePathCallback = filePathCallback

            //            takePhoto();
            startPhotoPicker()
            return true
        }

    }

    private var mPermissionHelper: PermissionHelper? = null

    override fun aidHandleMessage(what: Int, obj: Any?) {
        super.aidHandleMessage(what, obj)
        try {
            when (what) {

                loadurl_refresh_type -> mWebView.loadUrl(failUrl)
                linkRecipeDtl_refresh_type -> {
                    val intent = Intent(this, DetailsRecipeActivity::class.java)
                    intent.putExtra(DetailsRecipeActivity.ID, Integer.parseInt(obj.toString()))
                    startActivity(intent)
                }
                linkTheme_refresh_type -> {
                    val themeIntent = Intent(this, DetailsThemeActivity::class.java)
                    themeIntent.putExtra(DetailsThemeActivity.ID, Integer.parseInt(obj.toString()))
                    startActivity(themeIntent)
                }

            }
        } catch (e: Throwable) {
            LogUtils.e(TAG, e)
        }

    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_load_url)
        init()
    }

    private fun init() {
        try {
            backButton.setOnClickListener { finish() }
            mTitle.text = ""

            mUrl = intent.getStringExtra(URL)
            title = intent.getStringExtra(TITLE)

            if (!TextUtils.isEmpty(title)) {
                mTitle.text = title
            }

            initWebView()
            mWebView.loadUrl(mUrl)

            LogUtils.e(TAG, "url===>$mUrl")

        } catch (e: Throwable) {
            LogUtils.e(TAG, e)
        }

    }

    @SuppressLint("SetJavaScriptEnabled")
    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    private fun initWebView() {

        try {
            val settings = mWebView.settings
            settings.javaScriptEnabled = true
            settings.useWideViewPort = true
            settings.loadWithOverviewMode = true
            settings.layoutAlgorithm = WebSettings.LayoutAlgorithm.SINGLE_COLUMN
            settings.domStorageEnabled = true
            settings.mixedContentMode = android.webkit.WebSettings.MIXED_CONTENT_ALWAYS_ALLOW
            settings.allowContentAccess = true // 是否可访问Content Provider的资源，默认值 true
            settings.allowFileAccess = true // 是否可访问本地文件，默认值 true
            mWebView.addJavascriptInterface(MyJavaScriptInterface(), "myJsf")

            mWebView.webViewClient = webViewClient
            mWebView.webChromeClient = webChromeClient

        } catch (e: Throwable) {
            LogUtils.e(TAG, e)
        }

    }

    private inner class MyJavaScriptInterface {

        @JavascriptInterface
        fun linkRecipeDtl(id: Int) {

            if (id != 0) {
                aidsendMessage(linkRecipeDtl_refresh_type, id)
            }

            LogUtils.e(TAG, "linkRecipeDtl===>$id")
        }


        @JavascriptInterface
        fun reloadPage() {
            try {

                if (!TextUtils.isEmpty(failUrl)) {
                    aidsendMessage(loadurl_refresh_type, null)
                }


            } catch (e: Throwable) {
                LogUtils.e(TAG, e)
            }

        }

        @JavascriptInterface
        fun linkTheme(id: Int) {
            try {

                if (id != 0) {
                    aidsendMessage(linkTheme_refresh_type, id)
                }
            } catch (e: Throwable) {
                LogUtils.e(TAG, e)
            }

        }

    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>, grantResults: IntArray) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (mPermissionHelper != null) {
            mPermissionHelper!!.onRequestPermissionsResult(requestCode, permissions, grantResults)
        }
    }

    /**
     * 进入选择照片
     */
    private fun startPhotoPicker() {
        val intent = PhotoPickerIntent(this@LoadUrlActivity)
        intent.setPhotoCount(1)
        intent.setShowCamera(true)
        startActivityForResult(intent, PHOTO_REQUEST)
    }

    /**
     * 拍照
     */
    private fun takePhoto() {
        mPermissionHelper = PermissionHelper(this, arrayOf(Manifest.permission.CAMERA), 100)
        mPermissionHelper?.request(object : PermissionHelper.PermissionCallback {
            override fun onPermissionGranted() {
                fileUri = File(cacheDir.toString() + "/" + SystemClock.currentThreadTimeMillis() + ".jpg")
                imageUri = FileUriPermissionCompatUtil.adaptUri(this@LoadUrlActivity, fileUri)
                PhotoUtils.takePicture(this@LoadUrlActivity, imageUri, PHOTO_REQUEST)
            }

            override fun onIndividualPermissionGranted(grantedPermission: Array<String>) {

            }

            override fun onPermissionDenied() {

            }

            override fun onPermissionDeniedBySystem() {

            }
        })
    }


    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == PHOTO_REQUEST && resultCode == Activity.RESULT_OK) {
            if (data != null) {
                val photos = data.getStringArrayListExtra(PhotoPickerActivity.KEY_SELECTED_PHOTOS)
                val uri = FileUriPermissionCompatUtil.adaptUri(applicationContext, File(photos!![0]))
                AppTools.startPhotoZoom(this, uri)
            }
        } else if (requestCode == Comm.RESULT_ZOOM && resultCode == Activity.RESULT_OK) {
            try {
                hideProgressDialog()
                val options = BitmapFactory.Options()
                options.inPreferredConfig = Bitmap.Config.ARGB_4444
                options.inJustDecodeBounds = false
                options.outWidth = 200
                options.outHeight = 200
                //将Uri图片转换为Bitmap
                val photo = MediaStore.Images.Media.getBitmap(contentResolver, Global.uritempFile)
                val imgName = System.currentTimeMillis().toString()
                FileUtil.saveSDcardImage(photo, Comm.PATH_IMG, imgName)
                val filepath = Comm.PATH_IMG + "/" + imgName + ".png"
                showProgressDialog()
                if (null == mUploadMessage && null == mFilePathCallback) {
                    return
                }
                if (mFilePathCallback != null) {
                    onActivityResultAboveL(requestCode, filepath)
                } else if (mUploadMessage != null) {
                    mUploadMessage!!.onReceiveValue(FileUriPermissionCompatUtil.adaptUri(applicationContext, File(filepath)))
                    mUploadMessage = null
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }

        } else if (requestCode == VIDEO_REQUEST) {
            if (null == mUploadMessage && null == mFilePathCallback) {
                return
            }

            val result = if (data == null || resultCode != Activity.RESULT_OK) null else data.data
            if (mFilePathCallback != null) {
                if (resultCode == Activity.RESULT_OK && result != null) {
                    mFilePathCallback!!.onReceiveValue(arrayOf(result))
                } else {
                    mFilePathCallback!!.onReceiveValue(arrayOf())
                }
                mFilePathCallback = null

            } else if (mUploadMessage != null) {
                if (resultCode == Activity.RESULT_OK) {
                    mUploadMessage!!.onReceiveValue(result)
                } else {
                    mUploadMessage!!.onReceiveValue(Uri.EMPTY)
                }
                mUploadMessage = null
            }
        } else {
            if (mUploadMessage != null) {
                mUploadMessage!!.onReceiveValue(null)
                mUploadMessage = null
            }
            if (mFilePathCallback != null) {
                mFilePathCallback!!.onReceiveValue(null)
                mFilePathCallback = null
            }
        }
        hideProgressDialog()
    }

    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    private fun onActivityResultAboveL(requestCode: Int, imgUrl: String) {
        if (requestCode != Comm.RESULT_ZOOM || mFilePathCallback == null) {
            return
        }
        val uri = FileUriPermissionCompatUtil.adaptUri(applicationContext, File(imgUrl))
        mFilePathCallback!!.onReceiveValue(arrayOf(uri))
        mFilePathCallback = null
    }

    override fun onKeyDown(keyCode: Int, event: KeyEvent): Boolean {
        //如果按下的是回退键且历史记录里确实还有页面
        if (keyCode == KeyEvent.KEYCODE_BACK && mWebView.canGoBack()) {
            mWebView.goBack()
            return true
        }
        return super.onKeyDown(keyCode, event)
    }

    override fun onDestroy() {
        super.onDestroy()
        if (mWebView != null) {
            mWebView.webViewClient = null
            mWebView.webChromeClient = null
            mWebView.loadDataWithBaseURL(null, "", "text/html", "utf-8", null)
            mWebView.clearHistory()
            mWebView.destroy()
        }
    }


    companion object {

        private val TAG = LoadUrlActivity::class.java.simpleName
        private const val loadurl_refresh_type = 100
        private const val linkRecipeDtl_refresh_type = 101
        private const val linkTheme_refresh_type = 104

        const val URL = "url"
        const val TITLE = "title"
        private const val PHOTO_REQUEST = 100
        private const val VIDEO_REQUEST = 120
    }
}
