package com.gfeng.daydaycook.activity

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.os.CountDownTimer
import android.support.v4.content.ContextCompat
import android.text.Editable
import android.text.TextUtils
import android.text.TextWatcher
import android.view.LayoutInflater
import android.view.View
import android.view.WindowManager
import cn.sharesdk.framework.Platform
import cn.sharesdk.framework.PlatformActionListener
import cn.sharesdk.framework.ShareSDK
import cn.sharesdk.sina.weibo.SinaWeibo
import cn.sharesdk.tencent.qq.QQ
import cn.sharesdk.wechat.friends.Wechat
import cn.sharesdk.wechat.utils.WechatClientNotExistException
import com.alibaba.fastjson.JSON
import com.alibaba.fastjson.TypeReference
import com.gfeng.daydaycook.BuildConfig
import com.gfeng.daydaycook.R
import com.gfeng.daydaycook.comm.Comm
import com.gfeng.daydaycook.logic.NotifyMgr
import com.gfeng.daydaycook.model.AccountModel
import com.gfeng.daydaycook.model.ResponseModel
import com.gfeng.daydaycook.util.*
import kotlinx.android.synthetic.main.activity_login.*

/**
 * Life Is Better With DayDayCook
 * author: HuangQiang
 * created on: 2018/8/27 14:58.
 * description: 优化登录流程（mainland=1 国内版）
 */
class LoginActivity : BaseActivity(), View.OnClickListener, TextWatcher {
    private val get_code_network_type = 201
    private val login_network_type = 202
    private val platform_type = 203
    private val weixin_tishi_type = 204
    private val register_network_type = 205
    private val LOGIN_TYPE_PHONE = 1
    private val LOGIN_TYPE_WECHAT = 2
    private val LOGIN_TYPE_SINA = 3
    private val LOGIN_TYPE_QQ = 4


    private var loginType = LOGIN_TYPE_PHONE


    private val countDownTimer = object : CountDownTimer(60000, 1000) {
        override fun onTick(millisUntilFinished: Long) {
            verifyTv.text = String.format(resources.getString(R.string.new_login_send_code), (millisUntilFinished / 1000).toString())
        }

        override fun onFinish() {

            verifyTv.text = resources.getString(R.string.reg_reset_send)
            verifyTv.setTextColor(ContextCompat.getColor(applicationContext, R.color.main_style_color))
            verifyTv.isClickable = true

        }
    }


    private val plListener = object : PlatformActionListener {

        override fun onCancel(platform: Platform, arg1: Int) {
            hideProgressDialog()
            LogUtils.info("onCancel  platform==" + platform.toString() + " arg1==" + arg1)
        }

        override fun onComplete(platform: Platform, action: Int, map1: HashMap<String, Any>) {

            LogUtils.info("onComplete action==$action")
            hideProgressDialog()
            if (action == Platform.ACTION_USER_INFOR) {

                LogUtils.info(JSON.toJSONString(map1))
                aidsendMessage(platform_type, platform)

            }
        }

        override fun onError(platform: Platform, arg1: Int, e: Throwable) {
            LogUtils.info("onError  platform==" + platform.toString() + "  arg1==" + arg1 + "  throwable=>" + e)
            hideProgressDialog()
            if (e is WechatClientNotExistException) {
                aidsendMessage(weixin_tishi_type, null)
            }
            platform.removeAccount(true)
        }

    }

    override fun onCreate(savedInstanceState: Bundle?) {
        val view = LayoutInflater.from(this).inflate(R.layout.activity_login, null)
        setContentView(view)
        initView()
        super.onCreate(savedInstanceState)

        if (BuildConfig.DEBUG) {
            initDebugLogin()
        }
    }

    override fun initImmersionBar() {
        super.initImmersionBar()
        mImmersionBar.keyboardEnable(true, WindowManager.LayoutParams.SOFT_INPUT_ADJUST_PAN).init()
    }

    private fun initDebugLogin() {
        findViewById<View>(R.id.logoIv).setOnClickListener {
            val loginMap = HashMap<String, Any>()
            loginMap["userName"] = 18772937707
            loginMap["dynamicLogin"] = 0
            loginMap["type"] = 1
            loginMap["password"] = MD5.getMD5("111111")
            sendHttp(object : TypeReference<AccountModel>() {}.type, Comm.login, loginMap, login_network_type)
            showProgressDialog()
        }

    }

    private fun initView() {
        initToolbar()
        verifyTv.setOnClickListener(this)
        goButton.setOnClickListener(this)
        wechatButton.setOnClickListener(this)
        weiboButton.setOnClickListener(this)
        qqButton.setOnClickListener(this)
        telEdit.addTextChangedListener(this)
        pwdEdit.addTextChangedListener(this)

    }

    private fun initToolbar() {
        back.setOnClickListener { finish() }
    }

    override fun afterTextChanged(s: Editable?) {

    }

    override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {
    }

    override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {

        if (pwdEdit.length() > 0 && telEdit.length() > 0) {
            goButton.isClickable = true
            goButton.setBackgroundResource(R.drawable.background_newlogin_gobutton_orange)
        } else {
            goButton.isClickable = false
            goButton.setBackgroundResource(R.drawable.background_newlogin_gobutton_gray)
        }
    }


    override fun aidHandleMessage(what: Int, obj: Any?) {
        super.aidHandleMessage(what, obj)
        when (what) {
            R.id.verifyTv -> {

                val str = telEdit.text.toString().trim { it <= ' ' }.replace(" ", "")

                if (TextUtils.isEmpty(str)) {
                    NotifyMgr.showToastWithDialog(resources.getString(R.string.new_login_over_mobile), 0)
                } else {
                    if (Utils.isMobileNO(str)) {
                        val codeMap = HashMap<String, Any>()
                        codeMap["username"] = str
                        codeMap["type"] = 2
                        codeMap["isNew"] = 1
                        sendHttp(null, Comm.getRegCode, codeMap, get_code_network_type)
                        showProgressDialog()
                    } else {
                        NotifyMgr.showToastWithDialog(resources.getString(R.string.new_login_mobile_error), 0)

                    }
                }

            }
            R.id.goButton -> {
                val tel = telEdit.text.toString().trim { it <= ' ' }.replace(" ", "")
                if (!Utils.isMobileNO(tel)) {
                    NotifyMgr.showToastWithDialog(resources.getString(R.string.new_login_mobile_error), 0)
                    return
                }
                startCustomVerify(LOGIN_TYPE_PHONE)
            }
            R.id.wechatButton -> {
                startCustomVerify(LOGIN_TYPE_WECHAT)
            }
            R.id.weiboButton -> {
                startCustomVerify(LOGIN_TYPE_SINA)
            }
            R.id.qqButton -> {
                startCustomVerify(LOGIN_TYPE_QQ)
            }
            weixin_tishi_type -> {
                NotifyMgr.showShortToast(getString(R.string.wechat_client_inavailable))
            }
            platform_type -> {
                if (obj == null) {
                    return
                }

                val platform = obj as Platform
                val db = platform.db
                if (db != null) {
                    LogUtils.info("getName=" + platform.name)
                    LogUtils.info("getExpiresIn==" + db.expiresIn)
                    LogUtils.info("getExpiresTime==" + db.expiresTime)
                    LogUtils.info("getPlatformNname==" + db.platformNname)
                    LogUtils.info("getPlatformVersion==" + db.platformVersion)
                    LogUtils.info("getToken==" + db.token)
                    LogUtils.info("getTokenSecret==" + db.tokenSecret)
                    LogUtils.info("getUserGender==" + db.userGender)
                    LogUtils.info("getUserIcon==" + db.userIcon)
                    LogUtils.info("getUserId==" + db.userId)
                    LogUtils.info("getUserName==" + db.userName)
                } else {
                    return
                }

                val registerMap = HashMap<String, Any>()
                registerMap["userName"] = db.userId
                registerMap["nickName"] = db.userName
                registerMap["type"] = 1
                registerMap["image"] = db.userIcon
                registerMap["password"] = db.userId
                registerMap["sourceId"] = PhoneUtil.getInstance(this).getMetaData("UMENG_CHANNEL")

                when {
                    platform.name == SinaWeibo.NAME -> {
                        registerMap["loginType"] = 3 // loginType //登录类型 1客户端登录 2微信登录3微博登录 4 QQ登录 5

                    }
                    platform.name == Wechat.NAME -> {
                        val unionId = db.get("unionid")
                        LogUtils.info("unionId=$unionId")

                        registerMap["unionId"] = unionId
                        registerMap["loginType"] = 2

                    }
                    platform.name == QQ.NAME -> {
                        registerMap["loginType"] = 4

                    }
                }
                sendHttp(object : TypeReference<AccountModel>() {}.type, Comm.registered, registerMap, register_network_type)
                showProgressDialog()
            }

            ON_SUCCESS -> {
                hideProgressDialog()
                if (obj == null) {
                    return
                }

                val responseModel = obj as ResponseModel

                if (responseModel.code != Comm.CODE_200) {

                    NotifyMgr.showToastWithDialog(responseModel.data.toString(), 0)
                    return
                }
                when (responseModel.type) {

                    get_code_network_type -> {

                        countDownTimer.start()
                        verifyTv.isClickable = false
                        verifyTv.setTextColor(ContextCompat.getColor(this, R.color.normal_gray))

                    }
                    login_network_type -> {

                        val accountModel = responseModel.data as AccountModel
                        NotifyMgr.showToastWithDialog(getString(R.string.login_success), 1)
                        AccountHelper.handleAccountModel(this, accountModel)
                        setResult(Activity.RESULT_OK)
                        finish()


                    }
                    register_network_type -> {

                        val accountModel = responseModel.data as AccountModel
                        if (accountModel.needBind) {
                            val intent = Intent(this, BindPhoneActivity::class.java)
                            intent.putExtra(BindPhoneActivity.USER_ID, accountModel.id)
                            intent.putExtra(BindPhoneActivity.USER_UNIQUE_ID, accountModel.userUniqueId)
                            startActivityForResult(intent, 100)
                        } else {
                            NotifyMgr.showToastWithDialog(getString(R.string.login_success), 1);
                            AccountHelper.handleAccountModel(this, accountModel)
                            setResult(Activity.RESULT_OK)
                            finish()
                        }
                    }
                }
            }

            ON_FAILED -> {

            }
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if(requestCode == 100 && resultCode == Activity.RESULT_OK){
            setResult(Activity.RESULT_OK)
            finish()
        }
    }

    private fun doLogin() {
        val tel = telEdit.text.toString().trim { it <= ' ' }.replace(" ", "")
        val pwd = pwdEdit.text.toString().trim { it <= ' ' }.replace(" ", "")
        if (!Utils.isMobileNO(tel)) {
            NotifyMgr.showToastWithDialog(resources.getString(R.string.new_login_mobile_error), 0)
            return
        }

        val loginMap = HashMap<String, Any>()
        loginMap["userName"] = tel
        loginMap["dynamicLogin"] = 1
        loginMap["type"] = 1
        loginMap["code"] = pwd
        loginMap["password"] = tel
        loginMap["sourceId"] = PhoneUtil.getInstance(this).getMetaData("UMENG_CHANNEL")
        sendHttp(object : TypeReference<AccountModel>() {}.type, Comm.loginAndRegistered, loginMap, login_network_type)
        showProgressDialog()
    }

    /**
     * 开始验证  自定义api1及自定义api2示例
     * @param type 1 手机登录 2微信 3QQ登录 4 Facebook
     *
     */
    private fun startCustomVerify(type: Int) {
        loginType = type
        login()
    }

    /**
     * 急验证完成后完成登录或组册操作
     */
    private fun login() {
        when (loginType) {
            LOGIN_TYPE_PHONE -> {
                doLogin()
            }
            LOGIN_TYPE_WECHAT -> {
                showProgressDialog()
                val weChatPlatform = ShareSDK.getPlatform(Wechat.NAME)
                weChatPlatform.removeAccount(true)
                weChatPlatform.platformActionListener = plListener
                weChatPlatform.showUser(null)
            }
            LOGIN_TYPE_SINA -> {
                showProgressDialog()
                val weiBoPlatform = ShareSDK.getPlatform(SinaWeibo.NAME)
                weiBoPlatform.platformActionListener = plListener
                weiBoPlatform.removeAccount(true)
                weiBoPlatform.showUser(null)
            }
            LOGIN_TYPE_QQ -> {
                showProgressDialog()
                val qqPlatform = ShareSDK.getPlatform(QQ.NAME)
                qqPlatform.platformActionListener = plListener
                qqPlatform.removeAccount(true)
                qqPlatform.showUser(null)
            }
        }
    }


}