package com.gfeng.daydaycook.activity


import android.content.Intent
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.provider.Settings
import android.text.TextUtils
import android.view.View
import cn.jpush.android.api.JPushInterface
import com.gfeng.daydaycook.R
import com.gfeng.daydaycook.comm.Comm
import com.gfeng.daydaycook.comm.Global
import com.gfeng.daydaycook.discover.entity.TopContentLikeModel
import com.gfeng.daydaycook.logic.NotifyMgr
import com.gfeng.daydaycook.model.ResponseFailModel
import com.gfeng.daydaycook.model.ResponseModel
import com.gfeng.daydaycook.util.*
import com.gfeng.daydaycook.widget.ToastDialog
import com.google.gson.reflect.TypeToken
import kotlinx.android.synthetic.main.activity_set.*
import kotlinx.android.synthetic.main.top_layout.*
import java.io.File


/**
 * 设置界面
 */

open class SetActivity : BaseActivity() {


    private var shareUrl: String? = null
    private var shareTitle: String? = null
    private var shareContent: String? = null

    private var updateCheck: UpdateCheck? = null

    override fun aidHandleMessage(what: Int, obj: Any?) {
        super.aidHandleMessage(what, obj)
        try {
            when (what) {
                update_dialog -> {
                    val url = obj as String?
                    if (TextUtils.isEmpty(url)) {
                        return
                    }
                    val urlIntent = Intent()
                    urlIntent.action = "android.intent.action.VIEW"
                    val updateUri = Uri.parse(url)
                    urlIntent.data = updateUri
                    startActivity(urlIntent)
                }
                R.id.signoutButton -> {

                    //                    PushManager pushManager = PushManager.getInstance();
                    //                    pushManager.unBindAlias(this, "android_new_" + Global.currentAccountModel.getUserUniqueId(), true);
                    //jpush解除绑定发送广播调用接口
                    JPushInterface.deleteAlias(this, 1)
                    unBindJPush()
                }
                R.id.clearLayout ->

                    showAlertDialog(getString(R.string.prompt), getString(R.string.set_clear_cache_dialog), getString(R.string.clear), getString(R.string.cancel), clear_cache_refresh_type, BaseActivity.CUSTOM_DISMISS, null, null)
                clear_cache_refresh_type -> {
                    //清除应用内存缓存
                    //                    DataCleanManager.clearAllCache(Global.mContext);
                    //清除SD卡内存缓存
                    cacheSizeTextView.text = "0K"
                    /*
                     * 清空开屏广告信息
                     */
                    Utils.setLocalSaveStr(Global.mContext, Comm.SAVEFILE, Comm.SP_KEY_AD, "")
                    FileUtil2.deleteFilesByDirectory(File(Comm.PATH_DDC))
                }
                R.id.msgOnOffLayout -> switchCompat.toggle()
                R.id.shareLayout -> {
                    val shareHelper = ShareHelper.Builder(this@SetActivity)
                            .setShareTitle(shareTitle)
                            .setShareDes(shareContent)
                            .setShareImgUrl(Comm.share_default)
                            .setShareUrl(shareUrl)
                            .build()
                    shareHelper.startShare(window.decorView)
                }
                R.id.updateLayout -> {
                    updateCheck = UpdateCheck(this)
                    updateCheck?.setCheckVersionMode(UpdateCheck.ALWAYS)
                    updateCheck?.checkUpdate()
                    showProgressDialog()
                }

                R.id.aboutLayout ->
                    ShopWebActivity.goToPage(this, "关于", Global.webHost + Comm.about_address)

                ON_SUCCESS -> {
                    if (obj == null) {
                        return
                    }
                    val responseModel = obj as ResponseModel?

                    hideProgressDialog()

                    if (responseModel!!.code != Comm.CODE_200) {
                        NotifyMgr.showShortToast(responseModel.data.toString())
                        return
                    }

                    when (responseModel.type) {
                        unbind_jpush_network_type ->
                            //0：程序异常，服务端绑定失败
                            //1：服务端绑定成功
                            //2：参数异常
                            if (responseModel.code != "1") {
                                AccountHelper.LoginOut(this)
                                finish()
                            } else {
                                ToastDialog.showToastDialog(this@SetActivity, R.drawable.icon_toast_message, "程序异常，登出失败，请稍候再试", 2000)
                            }
                    }
                }
                ON_FAILED -> {
                    //请求失败后重新请求解绑
                    val failModel = obj as ResponseFailModel?
                    when (failModel!!.type) {
                        unbind_jpush_network_type -> ToastDialog.showToastDialog(this@SetActivity, R.drawable.icon_toast_message, "当前网络不可用，请检查你的网络设置", 2000)
                    }
                }
            }
        } catch (e: Throwable) {
            LogUtils.e(TAG, e)
        }

    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_set)
        init()
    }

    fun init() {
        try {
            mTitle.setText(R.string.activity_set)
            shareUrl = Global.webHost + Comm.DOWNLOAD

            shareTitle = getString(R.string.set_share_tittle)
            shareContent = getString(R.string.set_share_description)
            backButton.setOnClickListener { finish() }

            shareLayout.setOnClickListener(this)
            signoutButton.setOnClickListener(this)
            clearLayout.setOnClickListener(this)
            updateLayout.setOnClickListener(this)
            aboutLayout.setOnClickListener(this)


            switchCompat.setOnClickListener { goNotificationSetting() }
            val sdCarSize = DataCleanManager.getFormatSize(DataCleanManager.getFolderSize(File(Comm.PATH_DDC)).toDouble())
            LogUtils.e(TAG, "sdCarSize=$sdCarSize")

            cacheSizeTextView.text = sdCarSize

            if (Global.currentAccountModel == null) {
                signoutButton.visibility = View.GONE
            }

        } catch (e: Throwable) {
            LogUtils.e(TAG, e)
        }

    }

    override fun onResume() {
        super.onResume()
        switchCompat.isChecked = Utils.isNotificationsEnabled()

    }

    private fun goNotificationSetting() {
        try {
            val intent = Intent()
            when {
                Build.VERSION.SDK_INT >= 26 -> {
                    intent.action = Settings.ACTION_APPLICATION_DETAILS_SETTINGS
                    intent.data = Uri.fromParts("package", packageName, null)
                }
                Build.VERSION.SDK_INT > 24 -> {
                    intent.action = "android.settings.APP_NOTIFICATION_SETTINGS"
                    intent.putExtra("app_package", packageName)
                }
                Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP -> {
                    intent.action = "android.settings.APP_NOTIFICATION_SETTINGS"
                    intent.putExtra("app_package", packageName)
                    intent.putExtra("app_uid", applicationInfo.uid)
                }
                Build.VERSION.SDK_INT == Build.VERSION_CODES.KITKAT -> {
                    intent.action = Settings.ACTION_APPLICATION_DETAILS_SETTINGS
                    intent.addCategory(Intent.CATEGORY_DEFAULT)
                    intent.data = Uri.parse("package:$packageName")
                }
                else -> return
            }
            startActivity(intent)
        } catch (throwable: Throwable) {
            throwable.printStackTrace()
        }


    }

    /**
     * 极光用户关系解绑
     */
    private fun unBindJPush() {
        val jsonObject = com.alibaba.fastjson.JSONObject()
        jsonObject["platform"] = "1"
        jsonObject["registrationId"] = JPushInterface.getRegistrationID(applicationContext)
        jsonObject["mobile"] = Global.currentAccountModel.getUserName()
        LogUtils.d("JPushInterface", "Global.currentAccountModel.getUserName()=" + Global.currentAccountModel.getUserName())
        httpPost(object : TypeToken<TopContentLikeModel>() {}.type, Global.ucHost, Comm.userUnBind, jsonObject, unbind_jpush_network_type)
    }

    companion object {

        private val TAG = SetActivity::class.java.simpleName

        private const val clear_cache_refresh_type = 100
        private const val update_dialog = 103
        private const val unbind_jpush_network_type = 107//绑定极光推送
    }
}
