package com.gfeng.daydaycook.activity

import android.Manifest
import android.content.Context
import android.content.Intent
import android.graphics.drawable.ColorDrawable
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.support.v4.content.ContextCompat
import android.text.TextUtils
import android.util.TypedValue
import android.view.*
import android.widget.PopupWindow
import com.gfeng.daydaycook.R
import com.gfeng.daydaycook.comm.Comm
import com.gfeng.daydaycook.comm.Global
import com.gfeng.daydaycook.ui.BadgeView
import com.gfeng.daydaycook.util.LogUtils
import com.gfeng.daydaycook.util.PayHelper
import com.gfeng.daydaycook.util.ShareHelper
import com.gfeng.daydaycook.webview.OnJSWebViewLoadListener
import com.gfeng.daydaycook.webview.shopwebview.ShopWebJsInterface
import com.gfeng.daydaycook.webview.shopwebview.ShopWebUtils
import com.google.gson.JsonParser
import com.master.permissionhelper.PermissionHelper
import com.tencent.smtt.sdk.ValueCallback
import kotlinx.android.synthetic.main.activity_shop_web.*
import kotlinx.android.synthetic.main.top_layout.*
import org.json.JSONObject


/**
 * Created by HuangQiang on 18/6/14.
 */

class ShopWebActivity : BaseActivity() {

    private var url: String? = null
    private var type: Int = 0

    private val PHOTO_REQUEST = 100
    private val VIDEO_REQUEST = 120
    private var permissionHelper: PermissionHelper? = null
    private var msgDot: BadgeView? = null
    var isShowCar: Boolean = false//显示购物车数量
    var onPause: Boolean = false
    companion object {
        const val PAY_RESULT = 1
        @JvmField
        var ddctvFoodMaterialResponseDTOList: String = ""
        @JvmField
        var isShowCar: Boolean = false

        fun goToPage(context: Context, title: String, url: String?) {
            isShowCar = false
            goToPage(context, title, url, 0)
        }

        fun goToPage(context: Context, title: String, url: String?, ddctvFoodMaterialResponseDTOList: String) {
            this.ddctvFoodMaterialResponseDTOList = ddctvFoodMaterialResponseDTOList
            isShowCar =true
            goToPage(context, title, url, 0)

        }

        fun goToPage(context: Context, title: String, url: String?, type: Int) {
            if (TextUtils.isEmpty(url)) return
            val intent = Intent(context, ShopWebActivity::class.java)
            intent.putExtra("title", title)
            intent.putExtra("url", url)
            intent.putExtra("type", type)
            intent.putExtra("isShowCar", isShowCar)
            context.startActivity(intent)
        }
    }


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_shop_web)
        initTitleView()
        initData()

        showProgressDialog(false)

        Global.mAppMgr.setActivityDataRefreshListener(this, Comm.shop_web_activity)

    }

    override fun onDestroy() {
        Global.mAppMgr.setActivityDataRefreshListener(null, Comm.shop_web_activity)
        parentLayout.removeAllViews()
        webView.destroy()
        super.onDestroy()
    }

    private fun initTitleView() {
        mTitle.text = ""
        backButton.setOnClickListener {
            onBackPressed()
        }
    }

    private fun initData() {
        val title = intent.getStringExtra("title")
        isShowCar = intent.getBooleanExtra("isShowCar",false)
        mTitle.text = title

        //加上购物车
        if(isShowCar){
            shareButton.visibility = View.VISIBLE
            shareButton.setImageResource(R.drawable.title_cart)
            shareButton.setOnClickListener {
                ShopWebActivity.goToPage(this, "购物车", Global.webHost + Comm.shop_cart)
            }
        }

        type = intent.getIntExtra("type", 0)

        url = intent.getStringExtra("url")
        webView.setOnJSWebViewLoadListener(object : OnJSWebViewLoadListener {
            override fun loadFail() {
            }

            override fun onPageFinish(url: String?) {
                if(!isShowCar){
                    hideProgressDialog()
                }
            }

            override fun onPageStart(url: String?) {
            }

            override fun onReceivedTitle(title: String?) {
                if (TextUtils.isEmpty(mTitle.text)) {
                    mTitle.text = title
                }
            }
        })
        webView.loadUrl(url)

    }

    fun setWebTitle(title: String) {
        mTitle.text = title
    }

    fun setProductNumber(totalCount: Int) {
        if(msgDot==null){
            msgDot = BadgeView(this, shareButton,22)
        }
        if (totalCount == 0) {
            if (msgDot!!.isShown) {
                msgDot!!.hide()
            }
            return
        }
        msgDot!!.badgePosition = BadgeView.POSITION_TOP_RIGHT
        msgDot!!.setBadgeMargin(12, 18)
        msgDot!!.setTextSize(TypedValue.COMPLEX_UNIT_SP, 9f)
        msgDot!!.text = if (totalCount > 99) "99+"  else " $totalCount "
        msgDot!!.setTextColor(-0x1)
        msgDot!!.show()// 红点
    }

    fun getParentLayout(): ViewGroup {
        return parentLayout
    }

    /**
     * 弹出在线支付框
     */
    fun showPayView(orderNo: String, jumpUrl: String, failsJump: Int) {

        val view = LayoutInflater.from(this).inflate(R.layout.layout_pay_method_choose, null, false)
        val mPopupWindow = PopupWindow(view, WindowManager.LayoutParams.MATCH_PARENT, WindowManager.LayoutParams.WRAP_CONTENT, true)
        mPopupWindow.setBackgroundDrawable(ColorDrawable(ContextCompat.getColor(this, R.color.ddc_share_bg)))
        mPopupWindow.isOutsideTouchable = true
        mPopupWindow.animationStyle = R.style.Animation_AppCompat_Dialog

        view.findViewById<View>(R.id.weChatTv).setOnClickListener {
            PayHelper.weChatHttpPost(this, orderNo, jumpUrl, failsJump)
            dismissPayView(mPopupWindow)

        }
        view.findViewById<View>(R.id.aliPayTv).setOnClickListener {
            PayHelper.aliHttpPost(this, orderNo, jumpUrl, failsJump)
            dismissPayView(mPopupWindow)
        }
        view.findViewById<View>(R.id.cancelTv).setOnClickListener {
            dismissPayView(mPopupWindow)
        }

        if (!mPopupWindow.isShowing) {
            mPopupWindow.showAtLocation(getParentLayout(), Gravity.BOTTOM, 0, 0)
        }
    }

    private fun dismissPayView(mPopupWindow: PopupWindow) {
        if (mPopupWindow.isShowing) {
            mPopupWindow.dismiss()
        }
    }

    fun showShareBtn(jsonObject: JSONObject) {
        shareButton.visibility = View.VISIBLE
        shareButton.setImageResource(R.drawable.title_share_black)
        shareButton.setOnClickListener {
            var title = ""
            var desc = ""
            var imgUrl = ""
            var linkUrl = ""

            try {
                if (jsonObject.has("title"))
                    title = jsonObject.getString("title")

                if (jsonObject.has("desc"))
                    desc = jsonObject.getString("desc")

                if (jsonObject.has("imgUrl"))
                    imgUrl = jsonObject.getString("imgUrl")

                if (jsonObject.has("linkUrl"))
                    linkUrl = jsonObject.getString("linkUrl")

                val shareHelper = ShareHelper.Builder(this)
                        .setShareTitle(title)
                        .setShareDes(desc)
                        .setShareImgUrl(imgUrl)
                        .setShareUrl(linkUrl)
                        .build()
                shareHelper.startShare(getParentLayout())
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
    }

    override fun aidHandleMessage(what: Int, obj: Any?) {
        PayHelper.handleMessage(this, what, obj)
    }

    override fun onResume() {
        super.onResume()
        LogUtils.info("mActivity=" + Global.mActivity + " this=" + this)
        if (Global.mActivity == this) {
            if (Global.navigateBackRefresh) {
                refreshView()
                Global.navigateBackRefresh = false
                Global.mActivity = null
            } else {
                refreshView()
                Global.mActivity = null
            }

        } else {
            if (Global.navigateBackRefresh) {
                refreshView()
                Global.navigateBackRefresh = false
            }
        }
        //中间页需要刷新
        if(isShowCar && onPause){
//            refreshView()
            val js = "window.localStorage.getItem('carts');"
            var jsUrl = "javascript:(function({ var localStorage = window.localStorage;localStorage.getItem(carts) })()"
            var str = null
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
                webView.evaluateJavascript(js, object : ValueCallback<String>{
                    override fun onReceiveValue(str: String?) {
                        LogUtils.e("onReceiveValue","onReceiveValue="+str)
                        if(!TextUtils.isEmpty(str))
                        {
                            try {
                                val parser = JsonParser()
                                val json = str!!.replace("\\", "")
                                val jarray = parser.parse(if(json.startsWith("\""))json.substring(1,json.length-1) else json).asJsonArray
                                var total = 0
                                for (i in 0 until jarray.size()){
                                    val jsonObj = jarray[i].asJsonObject
                                    total += jsonObj.get("cart").asInt
                                }
                                setProductNumber(total)
                                refreshView()
                            } catch (e: Exception) {
                                refreshView()
                            }

                        }
                    }
                })
            } else {
                refreshView()
            }

        }
        onPause = false
    }

    private fun refreshView() {
        webView.reload()
        showProgressDialog()
    }

    override fun onPause() {
        super.onPause()
        onPause = true
    }

    override fun onBackPressed() {
        if (type == PAY_RESULT) {
            ShopWebJsInterface.goCategory(webView)
        } else {
            finish()
//            if (webView.canGoBack()) {
//                webView.goBack()
//            } else {
//                finish()
//
//            }
        }
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>, grantResults: IntArray) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        permissionHelper?.onRequestPermissionsResult(requestCode, permissions, grantResults)
    }

    /**
     * 打开图片选择器
     */
    fun takePhoto(mUploadMessage: ValueCallback<Uri>, mFilePathCallback: ValueCallback<Array<Uri>>) {
        permissionHelper = PermissionHelper(this, arrayOf(Manifest.permission.CAMERA), 100)
        permissionHelper!!.request(object : PermissionHelper.PermissionCallback {
            override fun onPermissionGranted() {
                ShopWebUtils.takePhoto(this@ShopWebActivity, mUploadMessage, mFilePathCallback, PHOTO_REQUEST)
            }

            override fun onIndividualPermissionGranted(grantedPermission: Array<String>) {
            }

            override fun onPermissionDenied() {
            }

            override fun onPermissionDeniedBySystem() {
            }
        })

    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        ShopWebUtils.onActivityResult(this, requestCode, resultCode, data)
    }

}