package com.gfeng.daydaycook.activity

import android.content.pm.ActivityInfo
import android.os.Bundle
import android.support.v7.app.AlertDialog
import cn.jzvd.JZDataSource
import cn.jzvd.JZUserAction
import cn.jzvd.Jzvd
import cn.jzvd.JzvdStd
import com.gfeng.daydaycook.R
import com.gfeng.daydaycook.logic.NotifyMgr
import com.gfeng.daydaycook.model.CommonShareUrlModel
import com.gfeng.daydaycook.model.SearchModel
import com.gfeng.daydaycook.model.StepModel
import com.gfeng.daydaycook.model.VideoDetailModel
import com.gfeng.daydaycook.model.choice.TodaySeenDetailsBean
import com.gfeng.daydaycook.model.choice.VideoSegsModel
import com.gfeng.daydaycook.util.AppTools.isWiFiAvailable
import com.gfeng.daydaycook.util.LogUtils
import com.gfeng.daydaycook.util.ShareHelper
import com.gfeng.daydaycook.widget.jzvd.MyJzvdStd
import kotlinx.android.synthetic.main.activity_video.*
import java.util.*


open class VideoActivity : BaseActivity() {
    private var indexModel: SearchModel? = null
    private var videoDetailModel: VideoDetailModel? = null
    private var todaySeenDetailsBean: TodaySeenDetailsBean? = null
    private var stepModel: StepModel? = null
    private var dialog: AlertDialog? = null


    override fun aidHandleMessage(what: Int, obj: Any?) {

        try {
            when (what) {
                cancel_refresh_type -> finish()
                sure_refresh_type -> {
                    dialog?.dismiss()
                    initVideo(false)
                }
                R.id.cancelButton -> finish()
                toast_rerfresh_type -> NotifyMgr.showShortToast(obj.toString())
            }

        } catch (e: Throwable) {
            LogUtils.e(TAG, e)
        }

    }


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_video)
        init()
    }

    private fun init() {
        try {
            indexModel = intent.getSerializableExtra(DATA) as SearchModel?
            videoDetailModel = intent.getSerializableExtra(DATA3) as VideoDetailModel?
            stepModel = intent.getSerializableExtra(DATA4) as StepModel?
            csu = intent.getSerializableExtra(SHARE_MODEL) as CommonShareUrlModel?
            todaySeenDetailsBean = intent.getSerializableExtra(DATA_TODAY_SEEN_DETAILS) as TodaySeenDetailsBean?

            if (indexModel == null && videoDetailModel == null && stepModel == null && todaySeenDetailsBean == null) {
                NotifyMgr.showShortToast("传入参数错误")
                finish()
                return
            }

            if (isWiFiAvailable(this)) {
                initVideo(true)
            } else {
                showNoWifiDialog()

            }

            MyJzvdStd.FULLSCREEN_ORIENTATION = ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE


            MyJzvdStd.setJzUserAction { type, url, screen, objects ->
                LogUtils.e("onEvent", "type->" + type + "--url-->" + url.toString() + "--screen-->" + screen)
                if (type == JZUserAction.ON_AUTO_COMPLETE) {
                    if (csu != null) {
                        val shareHelper = ShareHelper.Builder(this@VideoActivity)
                                .setShareTitle(csu!!.title)
                                .setShareDes(csu!!.content)
                                .setShareImgUrl(csu!!.imageUrl)
                                .setShareUrl(csu!!.shareUrl)
                                .build()
                        shareHelper.startShare(window.decorView)
                    }


                }
            }


        } catch (e: Throwable) {
            LogUtils.e(TAG, e)
        }

    }

    private fun showNoWifiDialog() {
        try {

            val builder = AlertDialog.Builder(this, com.jiuli.library.R.style.AlertDialogStyle)
            builder.setCancelable(true)
            val title = getString(R.string.is_wifi_available_title)
            builder.setTitle(title)

            builder.setMessage(R.string.is_wifi_available_content)
                    .setNegativeButton(R.string.is_wifi_available_cancel) { dialog, which -> aidsendMessage(cancel_refresh_type, null) }.setPositiveButton(R.string.is_wifi_available_continue) { dialog, which -> aidsendMessage(sure_refresh_type, null) }
            dialog = builder.create()
            dialog?.show()
        } catch (var10: Throwable) {
            LogUtils.e(TAG, "error=$var10")
        }

    }


    private fun initVideo(isWifiAvailable: Boolean) {

        try {

            var title = ""
            var url = ""
            var segs: VideoSegsModel? = null


            if (indexModel != null) {
                title = indexModel!!.title
                segs = indexModel!!.segs
                url = indexModel!!.detailsUrl

            } else if (videoDetailModel != null) {
                title = videoDetailModel!!.title
                segs = videoDetailModel!!.segs
                url = videoDetailModel!!.videoUrl

            } else if (stepModel != null) {
                title = stepModel!!.videoTitle
                segs = stepModel!!.segs
                url = stepModel!!.videoUrl
            } else if (todaySeenDetailsBean != null) {

                title = todaySeenDetailsBean!!.title
                segs = null
                url = todaySeenDetailsBean!!.video
            }


            try {
                if (segs != null) {

                    //JZVideoPlayer.SAVE_PROGRESS = false;
                    //                    MyJzvdStd.FULLSCREEN_ORIENTATION = ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE;  //横向
                    MyJzvdStd.setVideoImageDisplayType(MyJzvdStd.VIDEO_IMAGE_DISPLAY_TYPE_FILL_SCROP)

                    val map = LinkedHashMap<String, String>()
                    map["1080p"] = segs.p1080[0].url
                    map["720p"] = segs.p720[0].url
                    map["SD"] = segs.SD[0].url
                    val objects = arrayOfNulls<Any>(2)
                    objects[0] = map
                    objects[1] = false//looping
                    val jzDataSource = JZDataSource(map, title)
                    jzDataSource.looping = true
                    jzDataSource.currentUrlIndex = 2
                    jzDataSource.headerMap["key"] = "value"//header
                    jz_video.setUp(jzDataSource, JzvdStd.SCREEN_WINDOW_NORMAL)
                    //                    Glide.with(this).loadCenterCrop(VideoConstant.videoThumbList[0]).into(mJzvdStd.thumbImageView);
                    jz_video.seekToInAdvance = 20000

                    //                    jz_video.setUp(objects, isWifiAvailable ? 0 : 1
                    //                            , JZVideoPlayerStandard.SCREEN_WINDOW_FULLSCREEN, title);
                } else {
                    jz_video.setUp(url,
                            title, JzvdStd.SCREEN_WINDOW_NORMAL)
                    //                    jz_video.setUp(url
                    //                            , JZVideoPlayerStandard.SCREEN_WINDOW_FULLSCREEN, title);
                }

            } catch (throwable: Throwable) {
                LogUtils.e(TAG, "throwable=" + throwable.localizedMessage)
                //                jz_video.setUp(url
                //                        , JZVideoPlayerStandard.SCREEN_WINDOW_FULLSCREEN, title);
                jz_video.setUp(url,
                        title, JzvdStd.SCREEN_WINDOW_NORMAL)
            }

            jz_video.startVideo()
            jz_video.startWindowFullscreen()


        } catch (e: Throwable) {
            LogUtils.e(TAG, e)
        }

    }


    override fun onDestroy() {
        super.onDestroy()
        MyJzvdStd.releaseAllVideos()
        MyJzvdStd.setJzUserAction(null)
    }

    override fun onResume() {
        super.onResume()
        try {
            Jzvd.goOnPlayOnResume()
        } catch (e: Exception) {
            e.printStackTrace()
        }

    }

    override fun onPause() {
        super.onPause()
        try {
            MyJzvdStd.releaseAllVideos()
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }


    override fun onBackPressed() {
        super.onBackPressed()
        try {
            MyJzvdStd.releaseAllVideos()
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    companion object {

        private val TAG = VideoActivity::class.java.simpleName
        val DATA = "data"
        val DATA3 = "data4"
        val DATA4 = "data5"
        val DATA_TODAY_SEEN_DETAILS = "data_today_seen_details"//今日十见详情
        val SHARE_MODEL = "csu"

        private val toast_rerfresh_type = 103
        private val sure_refresh_type = 104
        private val cancel_refresh_type = 105

        internal var csu: CommonShareUrlModel? = null
    }

}
