package com.gfeng.daydaycook.activity

import android.Manifest
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.text.TextUtils
import android.util.Log
import android.view.View
import android.widget.RelativeLayout
import com.gfeng.daydaycook.R
import com.gfeng.daydaycook.comm.Comm
import com.gfeng.daydaycook.comm.Global
import com.gfeng.daydaycook.model.ResponseModel
import com.gfeng.daydaycook.util.LogUtils
import com.gfeng.daydaycook.util.PermissionUtils
import com.gfeng.daydaycook.util.Utils
import com.master.permissionhelper.PermissionHelper
import kotlinx.android.synthetic.main.activity_welcome.*


class WelcomeActivity : BaseActivity() {
    private var mPermissionHelper: PermissionHelper? = null
    private var videoStart: String? = null
    private var advertJson: String? = null

    override fun aidHandleMessage(what: Int, obj: Any?) {
        when (what) {
            R.id.startBtn -> startMainActivity()
            PERMISSION_CLOSE -> finish()
            PERMISSION_OPEN -> requestStoragePermission()
            PERMISSION_OPEN_SET -> PermissionUtils.launchAppDetailsSettings()
            ON_SUCCESS -> {
                val responseModel = obj as ResponseModel
                when (responseModel.type) {

                }
            }
        }
    }

    private fun playVideo() {
        Utils.setLocalSaveStr(Global.mContext, Comm.SAVEFILE, Comm.VIDEOSTART, "1")
        videoLayout.visibility = View.VISIBLE

        val ll = mVideoView.layoutParams as RelativeLayout.LayoutParams
        ll.width = Global.mScreenWidth
        ll.height = Global.mScreenHeight
        mVideoView.layoutParams = ll

        mVideoView.setVideoURI(Uri.parse("android.resource://com.gfeng.daydaycook/" + R.raw.startvideo))
        mVideoView.start()
        mVideoView.setOnPreparedListener { mp ->
            mp.start()
            mp.isLooping = false
        }
        mVideoView.setOnCompletionListener {
            mVideoView.stopPlayback()
            mVideoView.seekTo(1)
            startMainActivity()
        }

        startBtn.setOnClickListener(this)
    }


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_welcome)
        if (intent.flags and Intent.FLAG_ACTIVITY_BROUGHT_TO_FRONT != 0) {
            finish()
        }
    }

    private fun initData() {
        videoStart = Utils.getLocalSave(Global.mContext, Comm.SAVEFILE, Comm.VIDEOSTART)
        advertJson = Utils.getLocalSave(Global.mContext, Comm.SAVEFILE, Comm.SP_KEY_AD)
        LogUtils.e(TAG, "advertJson=$advertJson")

        when {
            TextUtils.isEmpty(videoStart) -> playVideo()
            TextUtils.isEmpty(advertJson) -> startMainActivity()
            else -> startAdViewActivity()
        }
    }


    private fun startMainActivity() {

        LogUtils.e(TAG, "startMainActivity")
        val intent = Intent(this@WelcomeActivity, TabActivity::class.java)
        startActivity(intent)
        finish()
    }

    private fun startAdViewActivity() {

        LogUtils.e(TAG, "startAdViewActivity")
        val intent = Intent(this@WelcomeActivity, AdViewActivity::class.java)
        startActivity(intent)
        finish()
    }


    override fun onStart() {
        super.onStart()
        if (mPermissionHelper == null) {
            mPermissionHelper = PermissionHelper(this, arrayOf(Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.READ_EXTERNAL_STORAGE), 100)
            requestStoragePermission()
        } else {
            requestStoragePermission()
        }
    }

    private fun requestStoragePermission() {
        mPermissionHelper!!.request(object : PermissionHelper.PermissionCallback {
            override fun onPermissionGranted() {
                Log.e(TAG, "onPermissionGranted")
                initData()
            }

            override fun onIndividualPermissionGranted(grantedPermission: Array<String>) {
                Log.e(TAG, "onIndividualPermissionGranted" + grantedPermission.toString())

            }

            override fun onPermissionDenied() {
                Log.e(TAG, "onPermissionDenied")
                showAlertDialog("权限提醒", getString(R.string.storage_permission_rationale_message), "确定", "关闭应用", PERMISSION_OPEN, PERMISSION_CLOSE, null, null, false)

            }

            override fun onPermissionDeniedBySystem() {
                Log.e(TAG, "onPermissionDeniedBySystem")
                showAlertDialog("权限提醒", getString(R.string.storage_permission_denied_forever_message), "去设置", "关闭应用", PERMISSION_OPEN_SET, PERMISSION_CLOSE, null, null, false)

            }
        })
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>, grantResults: IntArray) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        mPermissionHelper?.onRequestPermissionsResult(requestCode, permissions, grantResults)

    }

    companion object {

        private val TAG = WelcomeActivity::class.java.simpleName

        private const val PERMISSION_CLOSE = 106
        private const val PERMISSION_OPEN = 107
        private const val PERMISSION_OPEN_SET = 108
    }

}
