package com.gfeng.daydaycook.fragment

import android.content.Intent
import android.os.Bundle
import android.text.TextUtils
import android.util.TypedValue
import android.view.View
import com.alibaba.fastjson.JSON
import com.alibaba.fastjson.JSONObject
import com.gfeng.daydaycook.R
import com.gfeng.daydaycook.activity.*
import com.gfeng.daydaycook.comm.Comm
import com.gfeng.daydaycook.comm.Global
import com.gfeng.daydaycook.discover.entity.UnreadAnnounceModel
import com.gfeng.daydaycook.logic.AppMgr
import com.gfeng.daydaycook.logic.NotifyMgr
import com.gfeng.daydaycook.model.FocusAndFansCountModel
import com.gfeng.daydaycook.model.HasMessageModel
import com.gfeng.daydaycook.model.ResponseModel
import com.gfeng.daydaycook.model.UserScoresModel
import com.gfeng.daydaycook.publish.MyPublishActivity
import com.gfeng.daydaycook.ui.BadgeView
import com.gfeng.daydaycook.util.GlideUtils
import com.gfeng.daydaycook.util.LogUtils
import com.gfeng.daydaycook.util.StringUtils
import com.google.gson.reflect.TypeToken
import com.umeng.analytics.MobclickAgent
import kotlinx.android.synthetic.main.fragment_user.*
import java.util.*


/**
 * Created by HuangQiang on 2019/5/21.
 */
class UserFragment : BaseFragment() {

    private var unreadAnnounceModel: UnreadAnnounceModel? = null
    private var hmm: HasMessageModel? = null

    private var announceCount: Int = 0

    private var msgDot: BadgeView? = null
    private var couponDot: BadgeView? = null
    private var fansDot: BadgeView? = null


    override fun aidHandleMessage(what: Int, obj: Any?) {
        super.aidHandleMessage(what, obj)
        try {
            when (what) {

                userinfo_refresh_type -> refreshUserInfo()

                ON_SUCCESS -> {
                    hideProgressDialog()
                    val model = obj as ResponseModel

                    if (model.code != Comm.CODE_200) {
                        NotifyMgr.showShortToast(model.data.toString())
                    }

                    when (model.type) {
                        get_user_balance ->
                            if (!TextUtils.isEmpty(model.jsonStr)) {
                                try {
                                    val resJson = JSON.parseObject(model.jsonStr)
                                    val data = resJson.getDouble("data")
                                    balanceNumTv.text = "¥ " + StringUtils.formatMoney(data)
                                } catch (e: Exception) {
                                    e.printStackTrace()
                                }

                            }
                        get_user_coupon_num ->
                            if (!TextUtils.isEmpty(model.jsonStr)) {
                                try {
                                    val jsonObject = JSON.parseObject(model.jsonStr)
                                    val data = jsonObject.getJSONObject("data")
                                    val total = data.getInteger("total")
                                    showCouponDotCount(total)
                                } catch (e: Exception) {
                                    e.printStackTrace()
                                }
                            }
                        get_unread_announce -> {
                            unreadAnnounceModel = model.data as UnreadAnnounceModel
                            if (unreadAnnounceModel != null && unreadAnnounceModel!!.data != null) {
                                announceCount = unreadAnnounceModel!!.data.anounceCount
                            }
                            showMsgDotCount(hmm)
                        }
                        get_user_score -> {
                            val usm = model.data as UserScoresModel
                            bonusNumTv.text = usm.data.usable.toString()
                        }
                        get_user_focus_fans -> {
                            val focusAndFansCountModel = model.data as FocusAndFansCountModel
                            attentionNumTv.text = focusAndFansCountModel.data.watcherCount.toString()
                            fansNumTv.text = focusAndFansCountModel.data.followerCount.toString()
                            showFansDotCount(focusAndFansCountModel.data.newFollowerCount)
                        }

                        new_message_refresh_type -> {

                            if (model.code != Comm.CODE_200) {
                                return
                            }
                            hmm = model.data as HasMessageModel

                            getUnreadAnnounce()
                        }

                        else -> {
                        }
                    }
                }

                ON_FAILED -> {
                }
            }
        } catch (e: Throwable) {
            LogUtils.e(TAG, e)
        }

    }

    override fun getFragmentViewLayout(): Int {
        return R.layout.fragment_user
    }


    override fun onResume() {
        super.onResume()
        //刷新数据
        refreshUserInfo()
    }


    override fun initImmersionBar() {
        super.initImmersionBar()
        mImmersionBar.titleBar(titleLayout).statusBarDarkFont(false).init()

    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        MobclickAgent.onEvent(activity, "我的")

        initUi()
    }


    private fun initUi() {

        Global.mAppMgr.setActivityDataRefreshListener(this, Comm.useractivity_refresh_type)

        msgDot = BadgeView(activity, messageLayout)
        couponDot = BadgeView(activity, couponLl)
        fansDot = BadgeView(activity, fansLayout)

        settingLayout.setOnClickListener {
            //设置
            startActivity(Intent(activity, SetActivity::class.java))
        }

        messageLayout.setOnClickListener {
            //消息
            if (Global.currentAccountModel == null) {
                AppMgr.login(activity)
            } else {
                val intent = Intent(activity, MessageActivity::class.java)
                intent.putExtra(MessageActivity.HAS_MESSAGE, hmm)
                startActivity(intent)
            }
        }

        userLayout.setOnClickListener {
            //帮助中心我的资料
            if (Global.currentAccountModel == null) {
                AppMgr.login(activity)
            } else
                startActivity(Intent(activity, UserInfoActivity::class.java))
        }

        attentionLayout.setOnClickListener {
            //关注
            if (Global.currentAccountModel == null) {
                AppMgr.login(activity)
            } else {
                val attentionIntent = Intent(activity, FocusOrFansActivity::class.java)
                val bundle = Bundle()
                bundle.putSerializable("type", 1)
                bundle.putSerializable("userId", Global.currentAccountModel.getId().toString() + "")
                attentionIntent.putExtras(bundle)
                startActivity(attentionIntent)
            }
        }

        fansLayout.setOnClickListener {
            //粉丝
            if (Global.currentAccountModel == null) {
                AppMgr.login(activity)

            } else {
                val fansIntent = Intent(activity, FocusOrFansActivity::class.java)
                val bundle1 = Bundle()
                bundle1.putSerializable("type", 2)
                bundle1.putSerializable("userId", Global.currentAccountModel.getId().toString() + "")
                fansIntent.putExtras(bundle1)
                startActivity(fansIntent)
            }
        }

        bonusLayout.setOnClickListener {
            //风车叶
            if (Global.currentAccountModel == null) {
                AppMgr.login(activity)
            } else
                ShopWebActivity.goToPage(activity!!, "风车叶", Global.webHost + Comm.MY_POINTS)
        }

        collectionLayout.setOnClickListener {
            //收藏
            if (Global.currentAccountModel == null) {
                AppMgr.login(activity)
            } else
                startActivity(Intent(activity, CollectionActivity::class.java))
        }

        couponLayout.setOnClickListener {
            if (Global.currentAccountModel == null) {
                AppMgr.login(activity)
            } else
                ShopWebActivity.goToPage(activity!!, "优惠券", Global.webHost + Comm.shop_coupon)
        }

        footprintLayout.setOnClickListener {
            //足迹
            if (Global.currentAccountModel == null) {
                AppMgr.login(activity)
            } else
                startActivity(Intent(activity, FooterActivity::class.java))
        }
        publishLayout.setOnClickListener {
            //我的发布
            if (Global.currentAccountModel == null) {
                AppMgr.login(activity)
            } else
                startActivity(Intent(activity, MyPublishActivity::class.java))
        }

        balanceLayout.setOnClickListener {
            if (Global.currentAccountModel == null) {
                AppMgr.login(activity)
            } else
                ShopWebActivity.goToPage(activity!!, "我的余额", Global.webHost + Comm.balance)
        }

        orderLayout.setOnClickListener {
            if (Global.currentAccountModel == null) {
                AppMgr.login(activity)
            } else
                ShopWebActivity.goToPage(activity!!, "我的订单", Global.webHost + Comm.shop_order)
        }

        courseLayout.setOnClickListener {
            if (Global.currentAccountModel == null) {
                AppMgr.login(activity)
            } else
                ShopWebActivity.goToPage(context!!, "课程预约", Global.webHost + Comm.course)
        }

        addressLayout.setOnClickListener {
            if (Global.currentAccountModel == null) {
                AppMgr.login(activity)
            } else
                ShopWebActivity.goToPage(activity!!, "地址管理", Global.webHost + Comm.shop_address)
        }

        helpLayout.setOnClickListener {
            //帮助中心
            startActivity(Intent(activity, QAListActivity::class.java))
        }

    }


    internal fun initData() {
        getUserBalance()
        getUserScores()
        getCouponNum()
        getUserFocusAndFans()
        initDot()
    }


    /**
     * 查询用户余额
     */
    private fun getUserBalance() {
        if (TextUtils.isEmpty(Global.getUserUniqueId())) {
            return
        }
        val jsonObject = JSONObject()
        jsonObject["session"] = Global.getUserUniqueId()
        httpGet(object : TypeToken<ResponseModel>() {}.type, Global.ucHost, Comm.getUserBalance, jsonObject, get_user_balance)

    }


    /**
     * 获取优惠券个数
     */
    private fun getCouponNum() {
        if (TextUtils.isEmpty(Global.getUserUniqueId())) {
            return
        }

        val jsonObject = JSONObject()
        jsonObject["session"] = Global.getUserUniqueId()
        httpGet(object : TypeToken<ResponseModel>() {}.type, Global.ucHost, Comm.getUserCouponNum, jsonObject, get_user_coupon_num)

    }

    /**
     * 获取风车页
     */
    private fun getUserScores() {
        if (TextUtils.isEmpty(Global.getUserUniqueId())) {
            return
        }

        val jsonObject = JSONObject()
        jsonObject["session"] = Global.getUserUniqueId()
        httpGet(object : TypeToken<UserScoresModel>() {}.type, Global.ucHost, Comm.CREDIT_DETAIL, jsonObject, get_user_score)
    }

    /**
     * 获取用户关注、粉丝数，新增粉丝
     */
    private fun getUserFocusAndFans() {
        if (TextUtils.isEmpty(Global.getUserUniqueId())) {
            return
        }
        val jsonObject = JSONObject()
        jsonObject["sessionId"] = Global.getUserUniqueId()
        httpPost(object : TypeToken<FocusAndFansCountModel>() {}.type, Global.tvHost, Comm.FOCUS_FANS_TOTAL, jsonObject, get_user_focus_fans)

    }

    private fun initDot() {

        if (Global.currentAccountModel == null) {
            return
        }
        val map = HashMap<String, Any>()
        sendHttp(object : TypeToken<HasMessageModel>() {}.type, Comm.hasMessage, map, new_message_refresh_type)
    }


    private fun refreshUserInfo() {
        if (Global.currentAccountModel != null) {
            GlideUtils.loadCircleCrop(Global.currentAccountModel.image, R.drawable.default_user, iconIv)
            if (!TextUtils.isEmpty(Global.currentAccountModel.nickName)) {
                nameTv.text = Global.currentAccountModel.nickName
            } else {
                nameTv.text = getString(R.string.input_nick_name)
            }
            initData()

        } else {
            iconIv.setImageResource(R.drawable.default_user)
            nameTv.text = getString(R.string.myprofile_login_reg)
            attentionNumTv.text = "0"
            fansNumTv.text = "0"
            bonusNumTv.text = "0"
            balanceNumTv.text = "¥ 0"
            hideMsgDotCount()
            showFansDotCount(0)
            showCouponDotCount(0)
        }
    }

    private fun showCouponDotCount(total: Int) {
        if (total <= 0) {
            if (couponDot!!.isShown) {
                couponDot!!.hide()
            }
            return
        }

        couponDot!!.badgePosition = BadgeView.POSITION_TOP_RIGHT
        couponDot!!.setBadgeMargin(6, 0)
        couponDot!!.setTextSize(TypedValue.COMPLEX_UNIT_SP, 10f)
        couponDot!!.text = if (total > 99) " 99+ " else " " + total.toString() + " "
        couponDot!!.setTextColor(-0x1)
        couponDot!!.show()
    }

    /**
     * 新增粉丝数
     *
     * @param total
     */
    private fun showFansDotCount(total: Int) {
        if (total <= 0) {
            if (fansDot!!.isShown) {
                fansDot!!.hide()
            }
            return
        }

        fansDot!!.badgePosition = BadgeView.POSITION_TOP_RIGHT
        fansDot!!.setBadgeMargin(6, 0)
        fansDot!!.setTextSize(TypedValue.COMPLEX_UNIT_SP, 10f)
        fansDot!!.text = if (total > 99) " 99+ " else " " + total.toString() + " "
        fansDot!!.setTextColor(-0x1)
        fansDot!!.show()

    }

    private fun showMsgDotCount(hmm: HasMessageModel?) {
        val totalCount: Int = if (hmm == null || hmm.totalCount == 0) {
            announceCount
        } else {
            hmm.totalCount + hmm.OrderCount + announceCount
        }
        if (totalCount == 0) {
            if (msgDot!!.isShown) {
                msgDot!!.hide()
            }
            return
        }

        msgDot!!.badgePosition = BadgeView.POSITION_TOP_RIGHT
        msgDot!!.setBadgeMargin(12, 24)
        msgDot!!.setTextSize(TypedValue.COMPLEX_UNIT_SP, 10f)
        msgDot!!.text = if (totalCount > 99) " 99+ " else " $totalCount "
        msgDot!!.setTextColor(-0x1)
        msgDot!!.show()

    }

    private fun hideMsgDotCount() {
        if (msgDot!!.isShown) {
            msgDot!!.hide()
        }
    }

    /**
     * 获取互动界面未读消息
     */
    private fun getUnreadAnnounce() {
        val jsonObject = JSONObject()
        jsonObject["contentId"] = ""
        jsonObject["requestId"] = ""
        jsonObject["sessionId"] = Global.getUserUniqueId()
        httpPost(object : TypeToken<UnreadAnnounceModel>() {}.type, Global.tvHost, Comm.count_unread_announce, jsonObject, get_unread_announce)
    }

    override fun onDestroy() {
        super.onDestroy()
        Global.mAppMgr.setActivityDataRefreshListener(null, Comm.useractivity_refresh_type)
    }

    companion object {
        private val TAG = UserFragment::class.java.simpleName

        const val userinfo_refresh_type = 101

        private const val new_message_refresh_type = 102//获取消息
        private const val get_user_score = 103//获取积分（风车叶）
        private const val get_user_balance = 104//查询用户余额
        private const val get_user_coupon_num = 105//查询用户优惠券数量
        private const val get_user_focus_fans = 106//获取关注数和粉丝数
        private const val get_unread_announce = 107//获取未读消息数
    }
}