package com.gfeng.daydaycook.main

import android.content.Intent
import android.os.Bundle
import android.text.TextUtils
import android.view.View
import android.view.animation.AnimationUtils
import cn.jzvd.Jzvd
import com.alibaba.fastjson.JSON
import com.alibaba.fastjson.JSONObject
import com.gfeng.daydaycook.BuildConfig
import com.gfeng.daydaycook.DaydayCookApplication
import com.gfeng.daydaycook.R
import com.gfeng.daydaycook.activity.BindPhoneActivity
import com.gfeng.daydaycook.activity.ChangeApiServerActivity
import com.gfeng.daydaycook.comm.Comm
import com.gfeng.daydaycook.comm.Global
import com.gfeng.daydaycook.control.choice.DetailTodaySeenActivity
import com.gfeng.daydaycook.ddctv.ShoppingListActivity
import com.gfeng.daydaycook.discover.entity.LikeModel
import com.gfeng.daydaycook.discover.entity.TopContentLikeModel
import com.gfeng.daydaycook.fragment.BaseFragment
import com.gfeng.daydaycook.logic.AppMgr
import com.gfeng.daydaycook.logic.NotifyMgr
import com.gfeng.daydaycook.main.model.*
import com.gfeng.daydaycook.main.view.*
import com.gfeng.daydaycook.model.AdModel
import com.gfeng.daydaycook.model.ResponseModel
import com.gfeng.daydaycook.model.choice.ChoiceAdModel
import com.gfeng.daydaycook.model.choice.ChoiceGroupBuyBean
import com.gfeng.daydaycook.search.SearchResultActivity
import com.gfeng.daydaycook.util.LogUtils
import com.gfeng.daydaycook.util.NetworkUtil
import com.gfeng.daydaycook.util.Utils
import com.gfeng.daydaycook.widget.jzvd.MyJzvdStd
import com.google.gson.reflect.TypeToken
import kotlinx.android.synthetic.main.fragment_main.*
import java.util.*

class MainFragment : BaseFragment() {

    private var mainBannerView: MainBannerView? = null
    private var categoryEntryView: CategoryEntryView? = null
    private var mainExperienceView: MainExperienceView? = null
    private var groupBuyBean: ChoiceGroupBuyBean? = null //每日团购数据
    private var limitedTimeGroupView: LimitedTimeGroupView? = null
    private var communitySelectionView: CommunitySelectionView? = null
    private var hotTagsView: HotTagsView? = null
    private var ddctvView: DDCTVView? = null
    private var mainSpecialView: MainSpecialView? = null
    private var mainEveryDayQuestionsView: MainEveryDayQuestionsView? = null
    private var moseHotTagsModel: MostHotTagsModel? = null

    private var logStatus = false// 当前的登录状态
    private var lastMyJzvdStd: MyJzvdStd? = null
    private var completeBizId: Int = 0
    private var changePosition = -1
    private var showGuideStr: String? = ""

    private val jsonObject: JSONObject
        get() {
            val jsonObject = JSONObject()
            jsonObject["sessionId"] = Global.getUserUniqueId()
            jsonObject["requestId"] = ""
            return jsonObject
        }


    override fun aidHandleMessage(what: Int, obj: Any?) {
        when (what) {
            USER_STATUS_REFRESH_DATA -> {
                getTopicListRecommend()
                findRecommendContent()
            }
            ON_SUCCESS -> {
                if (obj == null) {
                    return
                }
                refreshLayout.finishRefresh()
                val responseModel = obj as ResponseModel?
                val jsonStr = responseModel!!.jsonStr
                var resJson: JSONObject? = null
                var code = Comm.CODE_0
                if (!TextUtils.isEmpty(jsonStr)) {
                    try {
                        resJson = JSON.parseObject(jsonStr)
                        if (resJson!!.containsKey("code")) {
                            code = resJson.getInteger("code")!!
                        }

                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
                }

                if (code == Comm.CODE_10000) {
                    AppMgr.login(context)
                    return
                }

                if (code == Comm.CODE_10006) {
                    NotifyMgr.showShortToast(resources.getString(R.string.no_bind_phone))
                    val bindIntent = Intent(context, BindPhoneActivity::class.java)
                    bindIntent.putExtra(BindPhoneActivity.USER_ID, Global.currentAccountModel.id)
                    bindIntent.putExtra(BindPhoneActivity.USER_UNIQUE_ID, Global.currentAccountModel.userUniqueId)
                    startActivity(bindIntent)
                    return
                }

                if (responseModel.code != Comm.CODE_200) {
                    return
                }

                when (responseModel.type) {

                    Get_Banner_List -> {
                        val adList = responseModel.data as List<AdModel>
                        mainBannerView?.refresh(adList)
                    }
                    GET_MATERIAL_TYPE_LIST -> {
                        val categoryEntryModelList = responseModel.data as List<CategoryEntryModel>
                        categoryEntryView?.setData(categoryEntryModelList)
                    }
                    GET_GROUP_BUY -> {
                        //限时拼团
                        groupBuyBean = ChoiceGroupBuyBean.objectFromData(resJson!!.getJSONObject("data").toJSONString())
                        limitedTimeGroupView?.setData(groupBuyBean)
                    }
                    GET_CHOICE -> {
                        // 体验馆/精选
                        val choiceAdModel = ChoiceAdModel.objectFromData(resJson!!.getJSONObject("data").toJSONString())
                        mainExperienceView?.refresh(choiceAdModel.experienceStoreList)
                    }
                    GET_TOPIC_LIST_RECOMMEND -> {
                        val recommendModel = responseModel.data as RecommendModel
                        communitySelectionView!!.setData(recommendModel)
                    }
                    GET_HOT_TAGS -> if (hotTagsView != null) {
                        val hotTagsModel = responseModel.data as HotTagsModel
                        hotTagsView?.setData(hotTagsModel)
                    }
                    FIND_HOT_TAG -> {
                        moseHotTagsModel = responseModel.data as MostHotTagsModel
                        if (moseHotTagsModel == null) return
                        if (moseHotTagsModel!!.data == null) return
                        if (hotTagsView != null) {
                            hotTagsView!!.setHotTags(moseHotTagsModel!!.data)
                        }
                    }
                    GET_HOTTEST_TAG_LIST -> if (hotTagsView != null) {
                        val hottestTagProductModel = responseModel.data as HottestTagProductModel
                        hotTagsView!!.setProductData(hottestTagProductModel)
                    }
                    GET_QUESTIONS -> {
                        // 问卷信息
                        val questionsModel = responseModel.data as QuestionsModel
                        if (TextUtils.equals(questionsModel.code, "0000")) {
                            mainEveryDayQuestionsView!!.refresh(questionsModel)
                            mainSpecialView!!.addDailyQuiz(mainEveryDayQuestionsView)
                        }
                    }
                    FIND_TV_TOP_LIST -> {
                        val ddctvBannerInfo = responseModel.data as DDCTVInfo
                        if (ddctvBannerInfo.data.size == 0){
                            ddctvView?.visibility = View.GONE
                            return
                        }
                        ddctvView?.visibility = View.VISIBLE
                        ddctvView!!.setData(ddctvBannerInfo)
                    }
                    FIND_DAILY_SELECTION_LIST -> {
                        val specialModel = responseModel.data as SpecialModel
                        if (specialModel.code == "0000") {
                            mainSpecialView?.setData(specialModel.data)
                        }
                    }
                    GET_QUESTIONS_SUBMIT -> LogUtils.e(TAG, responseModel.data.toString())// 用户答题，目前不需要处理结果
                    else -> {
                        if (responseModel.data == null)
                            return
                        val topContentLikeModel = responseModel.data as TopContentLikeModel
                        val position: Int
                        if (responseModel.type > 400) {
                            //社区精选点赞
                            LogUtils.d("TopContentLikeModel", "community_selection_thumps_up")
                            position = responseModel.type - community_selection_thumps_up
                            communitySelectionView!!.setThumpsUpState(topContentLikeModel, position)

                        } else {
                            LogUtils.d("TopContentLikeModel", "hot_tags_thumps_up")
                            //热门标签点赞
                            if (hotTagsView != null) {
                                position = responseModel.type - hot_tags_thumps_up
                                hotTagsView!!.setThumpsUpState(topContentLikeModel, position)
                            }
                        }
                    }
                }
            }
            ON_FAILED -> refreshLayout?.finishRefresh()
        }
    }

    override fun getFragmentViewLayout(): Int {
        return R.layout.fragment_main
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        getLoginStatus()
        initView()
        Global.mAppMgr.setActivityDataRefreshListener(this, Comm.mainframent_refresh_type)
        showGuide()//显示动画
    }

    override fun initImmersionBar() {
        super.initImmersionBar()
        mImmersionBar.titleBarMarginTop(titleLayout).statusBarColor(R.color.white).init()
    }

    private fun initView() {
        initTitle()
        refreshLayout?.setOnRefreshListener { getData() }
        refreshLayout.autoRefresh()

        addBannerView()
        addDDCTVView()
        addMainCategoryView()
        addSpecialView()
        addCommunitySelection()
        addHotTags()
        addLimitedTimeGroup()
        addMainExperienceView()
    }

    override fun onResume() {
        super.onResume()
        viewVisible()
        val tempLoginStatus = !TextUtils.isEmpty(Global.getUserUniqueId())
        if (tempLoginStatus != logStatus) {// 表示登录状态改变了
            logStatus = tempLoginStatus
            getQuestion()
        }
    }


    /**
     * 初始化title
     */
    private fun initTitle() {

        searchIv.setOnClickListener { SearchResultActivity.goToPage(context!!) }
        animationView.setOnClickListener { hideGuide() }
        tipsIv.setOnClickListener { hideGuide() }

        // 小红点信息点击事件
        headLeftLayout.setOnClickListener {
            if (TextUtils.isEmpty(showGuideStr)) {
                hideGuide()
                return@setOnClickListener
            }
            val intent = Intent()
            intent.setClass(context!!, ShoppingListActivity::class.java)
            startActivity(intent)
        }

        //测试时切换服务器作用
        if (BuildConfig.DEBUG) {
            titleIv.setOnClickListener { startActivity(Intent(context, ChangeApiServerActivity::class.java)) }
        }

    }


    /**
     * DDCTV
     */
    private fun addDDCTVView() {
        ddctvView = DDCTVView(context)
        ddctvView?.setOnVideoPlayListener { myJzvdStd ->
            if (myJzvdStd == null) return@setOnVideoPlayListener
            myJzvdStd.onPlayListener = object : MyJzvdStd.OnPlayListener {
                override fun onPause() {

                }

                override fun onPlaying() {

                }

                override fun onStateAutoComplete() {
                    doPlayRate(myJzvdStd, true)
                    //WiFi环境下，视频播放完成后，自动播放下一个
                    if (NetworkUtil.isWifi(context)) {
                        if (ddctvView != null && ddctvView!!.changePosition < ddctvView!!.list.size - 1) {
                            ddctvView!!.playNextVideo()
                        } else {
                            myJzvdStd.reStartVideo()
                        }
                    }

                }
            }
            lastMyJzvdStd = if (lastMyJzvdStd == null) {
                myJzvdStd
            } else {
                doPlayRate(lastMyJzvdStd, false)
                myJzvdStd
            }
        }
        contentLayout.addView(ddctvView)
    }

    /**
     * 添加专题
     */
    private fun addSpecialView() {
        mainSpecialView = MainSpecialView(context)
        addMainEveryDayQuestionsView()
        mainSpecialView!!.addDailyQuiz(mainEveryDayQuestionsView)
        contentLayout?.addView(mainSpecialView)
    }

    /**
     * 每日问答
     */
    private fun addMainEveryDayQuestionsView() {
        mainEveryDayQuestionsView = MainEveryDayQuestionsView(context)
        mainEveryDayQuestionsView!!.callback = MainEveryDayQuestionsView.Callback { questionsModel, _ -> getQuestionSubmit(questionsModel) }
    }

    /**
     * 视屏完播率
     * bizId     视频id
     * pageName  页面名称 首页/分类/详情
     * videoType 视频类型 允许值: 'ddctv', 'ugc', 'cookbook'
     * playTime  播放时长 int 四舍五入
     * videoTime 视频时长
     */
    fun doPlayRate(myJzvdStd: MyJzvdStd?, isComplete: Boolean) {
        if (myJzvdStd == null) return
        if (myJzvdStd.jzDataSource == null) return
        val headerMap = myJzvdStd.jzDataSource.headerMap
        if (headerMap == null || headerMap.size <= 0) return
        val bizId = headerMap["bizId"] as Int
        val pageName = headerMap["pageName"] as String
        val videoType = headerMap["videoType"] as String
        var playTime = if (myJzvdStd.playingPosition > 0) myJzvdStd.playingPosition else myJzvdStd.currentPositionWhenPlaying
        val duration = if (myJzvdStd.duration > 0) myJzvdStd.duration else MyJzvdStd.getBeforeDuration()
        playTime = if (isComplete) duration / 1000 else playTime / 1000
        val videoTime = duration / 1000
        if (videoTime == 0L) return
        if (playTime < 1) {
            return
        }
        if (isComplete) {
            completeBizId = bizId
        }
        if (!isComplete && bizId == completeBizId) return

        val jsonObject = JSONObject()
        jsonObject["bizId"] = bizId
        jsonObject["pageName"] = pageName
        jsonObject["videoType"] = videoType
        jsonObject["playTime"] = playTime
        jsonObject["videoTime"] = videoTime
        httpPost(null, Global.mcnHost, Comm.getMcnURL(Comm.DO_PLAY_RATE), jsonObject, 999)
    }

    /**
     * 社区精选
     */
    private fun addCommunitySelection() {
        communitySelectionView = CommunitySelectionView(activity!!, this)
        contentLayout!!.addView(communitySelectionView)
    }

    /**
     * banner
     */
    private fun addBannerView() {
        mainBannerView = MainBannerView(context)
        contentLayout!!.addView(mainBannerView)
    }

    /**
     * 分类
     */
    private fun addMainCategoryView() {
        categoryEntryView = CategoryEntryView(context)
        contentLayout?.addView(categoryEntryView)
    }

    /**
     * 体验馆
     */
    private fun addMainExperienceView() {
        mainExperienceView = MainExperienceView(context!!)
        contentLayout?.addView(mainExperienceView)
    }

    /**
     * 限时拼团
     */
    private fun addLimitedTimeGroup() {
        limitedTimeGroupView = LimitedTimeGroupView(context)
        contentLayout?.addView(limitedTimeGroupView)
    }

    /**
     * 热门标签
     */
    private fun addHotTags() {
        hotTagsView = HotTagsView(this, context)
        contentLayout?.addView(hotTagsView)
    }

    /**
     * 获取数据
     */
    private fun getData() {
        getBannerList()
        getCategoryEntry()//菜品分类
        getGroupBuy()
        getExperience()
        findDailySelectionList()
        getTopicListRecommend()
        getHotTags()
        getQuestion()
        findRecommendContent()
        getDDCTVList()
    }

    /**
     * 精选推荐（专题）
     */
    private fun findDailySelectionList() {
        httpPost(object : TypeToken<SpecialModel>() {}.type, Global.mcnHost, Comm.getMcnURL(Comm.FIND_DAILY_SELECTION_LIST), jsonObject, FIND_DAILY_SELECTION_LIST)
    }

    /**
     * 获取Banner数据
     */
    private fun getBannerList() {
        val adMap = HashMap<String, Any>()
        sendHttp(object : TypeToken<List<AdModel>>() {}.type, Comm.listAds, adMap, Get_Banner_List)
    }

    /**
     * 菜品分类入口
     */
    private fun getCategoryEntry() {
        val jsonObject = JSONObject()
        jsonObject["session"] = Global.getUserUniqueId()
        httpSend(object : TypeToken<List<CategoryEntryModel>>() {}.type, Global.mcnHost, Comm.getMcnURL(Comm.FIND_MATERIAL_TYPE_LIST), jsonObject, GET_MATERIAL_TYPE_LIST)
    }

    /**
     * DDC TV(今日吃什么)
     */
    private fun getDDCTVList() {
        val jsonObject = JSONObject()
        jsonObject["session"] = Global.getUserUniqueId()
        httpPost(object : TypeToken<DDCTVInfo>() {}.type, Global.mcnHost, Comm.getMcnURL(Comm.FIND_TV_TOP_LIST), jsonObject, FIND_TV_TOP_LIST)
    }

    /**
     * 获取每日拼团列表
     */
    private fun getGroupBuy() {
        httpPost(object : TypeToken<ResponseModel>() {}.type, Global.cmsAdHost, Comm.GET_GROUP_BUY, jsonObject, GET_GROUP_BUY)
    }

    /**
     * 每日问答
     */
    private fun getQuestion() {
        val jsonObject = JSONObject()
        jsonObject["session"] = Global.getUserUniqueId()
        httpPost(object : TypeToken<QuestionsModel>() {}.type, Global.mcnHost, Comm.getMcnURL(Comm.GET_QUESTIONS), jsonObject, GET_QUESTIONS)
    }

    /**
     * 用户答题
     */
    private fun getQuestionSubmit(questionsModel: QuestionsModel) {
        val jsonObject = JSONObject()
        jsonObject["session"] = Global.getUserUniqueId()
        jsonObject["qaId"] = questionsModel.data.id
        jsonObject["selectOption"] = questionsModel.data.selectedKey
        httpPost(null, Global.mcnHost, Comm.getMcnURL(Comm.GET_QUESTIONS_SUBMIT), jsonObject, GET_QUESTIONS_SUBMIT)
    }

    /**
     * 体验馆
     */
    private fun getExperience() {
        httpPost(null, Global.cmsAdHost, Comm.GET_CHOICE, JSONObject(), GET_CHOICE)
    }

    /**
     * 社区精选
     */
    private fun getTopicListRecommend() {
        val jsonObject = JSONObject()
        jsonObject["page"] = 1
        jsonObject["rows"] = 6
        jsonObject["sessionId"] = Global.getUserUniqueId()
        httpPost(object : TypeToken<RecommendModel>() {}.type, Global.tvHost, Comm.topicListRecommend, jsonObject, GET_TOPIC_LIST_RECOMMEND)
    }

    /**
     * 获取首页热门标签接口
     */
    private fun getHotTags() {
        val jsonObject = JSONObject()
        jsonObject["session"] = Global.getUserUniqueId()
        httpPost(object : TypeToken<HotTagsModel>() {}.type, Global.mcnHost, Comm.getMcnURL(Comm.FIND_RECOMMEND_HOME), jsonObject, GET_HOT_TAGS)
    }

    /**
     * 获取推荐tag
     */
    private fun findRecommendContent() {
        val jsonObject = JSONObject()
        jsonObject["session"] = Global.getUserUniqueId()
        httpPost(object : TypeToken<MostHotTagsModel>() {}.type, Global.mcnHost, Comm.getMcnURL(Comm.find_Recommend_Content), jsonObject, FIND_HOT_TAG)
    }


    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        LogUtils.d("onActivityResult", "resultCode=$resultCode")
        if (data != null) {
            if (resultCode == DetailTodaySeenActivity.result_code_delete_content) {
                val deletePosition = data.getIntExtra("position", 0)
                if (TextUtils.equals(data.getStringExtra("dispathType"), "hottagsproductAdapter")) {
                    //                    hotTagsView.removeHotTagProduct(deletePosition);
                    if (hotTagsView != null)
                        hotTagsView!!.setHotTags(moseHotTagsModel!!.data)
                } else if (TextUtils.equals(data.getStringExtra("dispathType"), "CommunitySelectionAdapter")) {
                    if (communitySelectionView != null)
                        communitySelectionView!!.removeCommunitySelection(deletePosition)
                }
            } else {
                if (data.getSerializableExtra("likeModel") != null) {
                    val likeModel = data.getSerializableExtra("likeModel") as LikeModel
                    if (TextUtils.equals(data.getStringExtra("dispathType"), "hottagsproductAdapter")) {
                        //                        hotTagsView.refreshThumpsUpState(likeModel);
                        if (hotTagsView != null)
                            hotTagsView!!.setHotTags(moseHotTagsModel!!.data)
                    } else if (TextUtils.equals(data.getStringExtra("dispathType"), "CommunitySelectionAdapter")) {
                        if (communitySelectionView != null)
                            communitySelectionView!!.refreshThumpsUpState(likeModel)
                    }

                }
            }

        }
    }

    /**
     * 得到当前的登录状态
     *
     * @return
     */
    private fun getLoginStatus() {
        logStatus = !TextUtils.isEmpty(Global.getUserUniqueId())
    }


    override fun onPause() {
        LogUtils.e(TAG, "onPause")
        viewGone()
        super.onPause()
    }

    //界面显示获得
    private fun viewVisible() {
        if (soundOFF) {
            DaydayCookApplication.setVolume(0f, 0f)
        }
        if (ddctvView != null && ddctvView!!.ddctvAdapter != null && changePosition != -1) {
            ddctvView!!.changePosition = changePosition
            lastMyJzvdStd = ddctvView!!.ddctvAdapter.getViewByPosition(ddctvView!!.changePosition, R.id.jz_video) as MyJzvdStd?
        }
    }

    //界面隐藏操作
    private fun viewGone() {
        if (ddctvView != null) {
            changePosition = if (ddctvView!!.changePosition == -1) {
                0
            } else {
                ddctvView!!.changePosition
            }

        }
        doPlayRate(lastMyJzvdStd, false)
        Jzvd.releaseAllVideos()
        DaydayCookApplication.setVolume(1f, 1f)
    }

    override fun onDestroy() {
        super.onDestroy()
        Global.mAppMgr.setActivityDataRefreshListener(null, Comm.mainframent_refresh_type)
    }

    override fun onHiddenChanged(hidden: Boolean) {
        super.onHiddenChanged(hidden)
        LogUtils.i(TAG, "onHiddenChanged==$hidden")
        if (hidden) {
            viewGone()
        } else {
            viewVisible()
        }
    }

    /**
     * 隐藏引导
     */
    private fun hideGuide() {
        animationView.clearAnimation()
        animationView.visibility = View.GONE
        tipsIv.visibility = View.GONE
        showGuideStr = "1"
        Utils.setLocalSaveStr(Global.mContext, Comm.SAVEFILE, Comm.ADD_SHOP_LIST_HOME, "1")
    }

    /**
     * 显示买菜清单引导
     */
    private fun showGuide() {
        showGuideStr = Utils.getLocalSave(Global.mContext, Comm.SAVEFILE, Comm.ADD_SHOP_LIST_HOME)
        if (!TextUtils.isEmpty(showGuideStr)) {
            return
        }
        tipsIv.visibility = View.VISIBLE
        animationView.visibility = View.VISIBLE
        animationView.animation = AnimationUtils.loadAnimation(activity, R.anim.animation_home_add_list_guie)
        animationView.animation.start()
    }

    companion object {
        private val TAG = MainFragment::class.java.simpleName
        var soundOFF = true//控制静音

        const val USER_STATUS_REFRESH_DATA = 101//用户登陆后改变状态

        private const val GET_GROUP_BUY = 102//获取团购列表
        private const val Get_Banner_List = 103//获取Banner内容
        private const val GET_CHOICE = 104//体验馆
        private const val GET_TOPIC_LIST_RECOMMEND = 105//社区精选
        private const val GET_HOT_TAGS = 106//热门标签
        private const val FIND_HOT_TAG = 107//热门标签
        private const val GET_HOTTEST_TAG_LIST = 108//热门标签
        private const val FIND_TV_TOP_LIST = 109//ddc tv
        private const val FIND_DAILY_SELECTION_LIST = 110//精选推荐
        private const val GET_QUESTIONS = 111// 每日问答
        private const val GET_QUESTIONS_SUBMIT = 112// 用户答题
        private const val GET_MATERIAL_TYPE_LIST = 113//获取食谱分类列表

        const val hot_tags_thumps_up = 301 //热门标签点赞(不能随便改，请求结果处理地方有用)
        const val community_selection_thumps_up = 401//社区精选点赞(不能随便改，请求结果处理地方有用)
    }

}
