package com.gfeng.daydaycook.search

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.support.v4.content.ContextCompat
import android.view.View
import com.alibaba.fastjson.TypeReference
import com.gfeng.daydaycook.R
import com.gfeng.daydaycook.activity.BaseActivity
import com.gfeng.daydaycook.comm.Comm
import com.gfeng.daydaycook.comm.Global
import com.gfeng.daydaycook.model.ResponseModel
import com.gfeng.daydaycook.model.ScreeningNextModel
import com.gfeng.daydaycook.search.adapter.FilterTagAdapter
import com.zhy.view.flowlayout.FlowLayout
import com.zhy.view.flowlayout.TagFlowLayout
import kotlinx.android.synthetic.main.activity_filter.*
import kotlinx.android.synthetic.main.top_layout.*

/**
 * Life Is Better With DayDayCook
 * author: HuangQiang
 * created on: 2019/4/19 11:19.
 * description:搜索条件过滤类
 */
class FilterActivity : BaseActivity(), TagFlowLayout.OnTagClickListener {

    private val flavor_network_type = 101

    private var favorAdapter: FilterTagAdapter? = null
    private var skillAdapter: FilterTagAdapter? = null
    private var timeAdapter: FilterTagAdapter? = null
    private var favorList: MutableList<ScreeningNextModel> = mutableListOf()
    private var skillList: MutableList<ScreeningNextModel> = mutableListOf()
    private var timeList: MutableList<ScreeningNextModel> = mutableListOf()
    private var favorModel: ScreeningNextModel? = null
    private var skillModel: ScreeningNextModel? = null
    private var timeModel: ScreeningNextModel? = null

    companion object {

        var flavorSelectName = "flavorSelectName"
        var skillSelectName = "skillSelectName"
        var timeSelectName = "timeSelectName"

        fun goToPage(activity: Activity, favorModel: ScreeningNextModel?, skillModel: ScreeningNextModel?, timeModel: ScreeningNextModel?, requestCode: Int) {
            val intent = Intent(activity, FilterActivity::class.java)
            if (favorModel != null)
                intent.putExtra(flavorSelectName, favorModel)
            if (skillModel != null)
                intent.putExtra(skillSelectName, skillModel)
            if (timeModel != null)
                intent.putExtra(timeSelectName, timeModel)
            activity.startActivityForResult(intent, requestCode)
        }
    }


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_filter)
        initData()
        initView()
        getData()
    }

    private fun getData() {
        httpSend(object : TypeReference<List<ScreeningNextModel>>() {}.type, Global.mcnHost, Comm.getMcnURL(Comm.RECIPE_SEARCH_TECHNICS), null, flavor_network_type)
    }

    private fun initData() {
        if (intent.hasExtra(flavorSelectName))
            favorModel = intent.getSerializableExtra(flavorSelectName) as ScreeningNextModel
        if (intent.hasExtra(skillSelectName))
            skillModel = intent.getSerializableExtra(skillSelectName) as ScreeningNextModel
        if (intent.hasExtra(timeSelectName))
            timeModel = intent.getSerializableExtra(timeSelectName) as ScreeningNextModel

        formatList(favorList)
        formatList(skillList)
        formatList(timeList)
    }

    //数组前加上 不限
    private fun formatList(list: MutableList<ScreeningNextModel>) {
        list.clear()
        val defaultModel = ScreeningNextModel()
        defaultModel.name = getString(R.string.found_type_itme_any)
        list.add(defaultModel)
    }

    private fun initView() {
        backButton.visibility = View.GONE
        backTv.visibility = View.VISIBLE
        rightButton.visibility = View.VISIBLE
        shareButton.visibility = View.GONE
        rightButton.text = "完成"
        rightButton.setTextColor(ContextCompat.getColor(this, R.color.main_style_color))
        mTitle.text = "筛选"

        backTv.setOnClickListener {
            onBackPressed()
        }

        rightButton.setOnClickListener {
            val intent = Intent()
            if (favorModel != null)
                intent.putExtra(flavorSelectName, favorModel)
            if (skillModel != null)
                intent.putExtra(skillSelectName, skillModel)
            if (timeModel != null)
                intent.putExtra(timeSelectName, timeModel)

            setResult(RESULT_OK, intent)
            onBackPressed()
        }

        favorFlowLayout.setOnTagClickListener(this)
        skillFlowLayout.setOnTagClickListener(this)
        timeFlowLayout.setOnTagClickListener(this)

        favorFlowLayout.setOnSelectListener { selectPosSet ->
            val intArray = selectPosSet.toTypedArray()
            if (intArray.isNotEmpty() && intArray[0] != null && intArray[0] != 0) {
                favorModel = favorAdapter?.getItem(intArray[0])
                favorModel?.position = intArray[0]
            } else
                favorModel = null
        }
        skillFlowLayout.setOnSelectListener { selectPosSet ->
            val intArray = selectPosSet.toTypedArray()
            if (intArray.isNotEmpty() && intArray[0] != null && intArray[0] != 0) {
                skillModel = skillAdapter?.getItem(intArray[0])
                skillModel?.position = intArray[0]
            } else
                skillModel = null
        }
        timeFlowLayout.setOnSelectListener { selectPosSet ->
            val intArray = selectPosSet.toTypedArray()
            if (intArray.isNotEmpty() && intArray[0] != null && intArray[0] != 0) {
                timeModel = timeAdapter?.getItem(intArray[0])
                timeModel?.position = intArray[0]
            } else
                timeModel = null
        }

        favorAdapter = FilterTagAdapter(this, favorList)
        favorFlowLayout.adapter = favorAdapter
        favorAdapter?.setSelectedList(favorModel?.position ?: 0)

        skillAdapter = FilterTagAdapter(this, skillList)
        skillFlowLayout.adapter = skillAdapter
        skillAdapter?.setSelectedList(skillModel?.position ?: 0)

        timeAdapter = FilterTagAdapter(this, timeList)
        timeFlowLayout.adapter = timeAdapter
        timeAdapter?.setSelectedList(timeModel?.position ?: 0)
    }

    override fun aidHandleMessage(what: Int, obj: Any?) {
        super.aidHandleMessage(what, obj)
        when (what) {
            ON_SUCCESS -> {

                val responseModel = obj as ResponseModel
                if (responseModel.code != Comm.CODE_200) {
                    return
                }

                when (responseModel.type) {
                    flavor_network_type -> {

                        val list = responseModel.data as List<ScreeningNextModel>
                        if (list.isNotEmpty()) {
                            list.forEach { screeningNextModel ->
                                when (screeningNextModel.technologyId) {
                                    2 -> {
                                        formatList(favorList)
                                        favorList.addAll(screeningNextModel.vos)
                                        favorAdapter?.notifyDataChanged()
                                    }

                                    1 -> {
                                        formatList(skillList)
                                        skillList.addAll(screeningNextModel.vos)
                                        skillAdapter?.notifyDataChanged()
                                    }

                                    3 -> {
                                        formatList(timeList)
                                        timeList.addAll(screeningNextModel.vos)
                                        timeAdapter?.notifyDataChanged()
                                    }

                                }
                            }


                        }

                    }
                }
            }
            ON_FAILED -> {

            }

        }
    }

    override fun onTagClick(view: View?, position: Int, parent: FlowLayout): Boolean {
        for (i in 0 until parent.childCount) {
            parent.getChildAt(i).isClickable = i == position
        }
        return false
    }

}