package com.gfeng.daydaycook.search


import android.content.Intent
import android.os.Bundle
import android.support.v7.widget.RecyclerView
import android.support.v7.widget.StaggeredGridLayoutManager
import android.text.TextUtils
import android.view.View
import android.widget.TextView
import com.alibaba.fastjson.JSONObject
import com.alibaba.fastjson.TypeReference
import com.gfeng.daydaycook.R
import com.gfeng.daydaycook.comm.Comm
import com.gfeng.daydaycook.comm.Global
import com.gfeng.daydaycook.control.choice.DetailTodaySeenActivity
import com.gfeng.daydaycook.discover.entity.LikeModel
import com.gfeng.daydaycook.discover.entity.TopContentLikeModel
import com.gfeng.daydaycook.fragment.BaseFragment
import com.gfeng.daydaycook.logic.AppMgr
import com.gfeng.daydaycook.model.ResponseFailModel
import com.gfeng.daydaycook.model.ResponseModel
import com.gfeng.daydaycook.search.adapter.SearchContentResultAdapter
import com.gfeng.daydaycook.search.entity.SearchContentModel
import com.gfeng.daydaycook.util.LogUtils
import com.google.gson.reflect.TypeToken
import com.scwang.smartrefresh.layout.api.RefreshLayout
import com.scwang.smartrefresh.layout.listener.OnRefreshLoadMoreListener
import kotlinx.android.synthetic.main.fragment_search_recipe.*

/**
 * Life Is Better With DayDayCook
 * author: HuangQiang
 * created on: 2019/4/16 14:54.
 * description:搜索的UGC内容列表页
 */
class SearchContentFragment : BaseFragment() {
    private val TAG: String = "SearchContentFragment"

    private var name: String? = ""//搜索词
    private var pageIndex = 1//分页页码 从0开始

    private val SEARCH_REFRESH_TYPE = 101
    private val SEARCH_MORE_TYPE = 102
    private val network_thumps_up = 103


    companion object {
        fun getInstance(name: String): SearchContentFragment {
            val fragment = SearchContentFragment()
            val bundle = Bundle()
            bundle.putString("name", name)
            fragment.arguments = bundle
            return fragment
        }
    }

    override fun getFragmentViewLayout(): Int {
        return R.layout.fragment_search_content
    }


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        name = arguments?.getString("name") ?: ""
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initView()
    }

    private var adapter: SearchContentResultAdapter? = null

    private fun initView() {
        refreshLayout.setOnRefreshLoadMoreListener(object : OnRefreshLoadMoreListener {
            override fun onLoadMore(refreshLayout: RefreshLayout?) {
                LogUtils.e(TAG, "onLoadMore")
                getSearchData(++pageIndex, SEARCH_MORE_TYPE)
            }

            override fun onRefresh(refreshLayout: RefreshLayout?) {
                LogUtils.e(TAG, "onRefresh")
                refreshLayout?.setNoMoreData(false)//重新可以加载更多
                pageIndex = 1
                getSearchData(pageIndex, SEARCH_REFRESH_TYPE)
            }
        })

        refreshLayout.isEnableAutoLoadMore = true//滑动到底部自动加载更多
        refreshLayout.autoRefresh()//第一次加载数据

        val staggeredGridLayoutManager = object : StaggeredGridLayoutManager(2, StaggeredGridLayoutManager.VERTICAL) {
            override fun onLayoutChildren(recycler: RecyclerView.Recycler?, state: RecyclerView.State) {
                try {
                    super.onLayoutChildren(recycler, state)
                } catch (e: IndexOutOfBoundsException) {
                    e.printStackTrace()
                }

            }
        }
        recyclerView.layoutManager = staggeredGridLayoutManager
        adapter = SearchContentResultAdapter(R.layout.item_search_content)
        recyclerView.adapter = adapter

        adapter?.setOnItemClickListener { adapter, _, position ->
            val model: SearchContentModel.Data = adapter.data[position] as SearchContentModel.Data
            val intent = Intent(context, DetailTodaySeenActivity::class.java)
            intent.putExtra("businessCategoryId", model.businessCategoryId)
            intent.putExtra("contentId", model.id)
            intent.putExtra("position", position)
            startActivityForResult(intent, 100)
        }
        adapter?.setOnItemChildClickListener { adapter, view, position ->
            when (view.id) {
                R.id.ll_thumbs_up -> {
                    if (TextUtils.isEmpty(Global.getUserUniqueId())) {
                        AppMgr.login(activity)

                    } else {
                        val model: SearchContentModel.Data = adapter.data[position] as SearchContentModel.Data
                        thumpsUp(model)

                        var likeCount = model.likeCount
                        if (model.isLike == 1) {
                            model.isLike = 0

                            if (likeCount > 0) {
                                likeCount -= 1
                            }
                            model.likeCount = likeCount
                        } else {
                            model.isLike = 1
                            likeCount += 1
                            model.likeCount = likeCount
                        }

                        adapter.notifyItemChanged(position)
                    }
                }
            }
        }
    }

    /**
     * 获取搜索结果
     * @param index Int
     */
    private fun getSearchData(index: Int, refreshType: Int) {
        LogUtils.e(TAG, "search content index=$index")

        val jsonObject = JSONObject()
        jsonObject["q"] = name
        jsonObject["pageIndex"] = index
        jsonObject["pageRow"] = 20
        jsonObject["sessionId"] = Global.getUserUniqueId()
        httpPost(object : TypeReference<SearchContentModel>() {}.type, Global.tvHost, Comm.contentSearch, jsonObject, refreshType)
    }


    /**
     * 点赞
     */
    private fun thumpsUp(model: SearchContentModel.Data) {
        val jsonObject = JSONObject()
        jsonObject["businessCategoryId"] = model.businessCategoryId
        jsonObject["contentId"] = model.id
        jsonObject["sessionId"] = Global.getUserUniqueId()
        jsonObject["state"] = model.isLike
        httpPost(object : TypeToken<TopContentLikeModel>() {}.type, Global.tvHost, Comm.thumpsUp, jsonObject, network_thumps_up)
    }

    override fun aidHandleMessage(what: Int, obj: Any?) {
        super.aidHandleMessage(what, obj)
        when (what) {
            ON_SUCCESS -> {

                val responseModel = obj as ResponseModel

                when (responseModel.type) {
                    SEARCH_REFRESH_TYPE -> {
                        refreshLayout.finishRefresh()

                        val searchContentModel = responseModel.data as SearchContentModel
                        val dataList = searchContentModel.data
                        if (dataList.isNotEmpty())
                            adapter?.setNewData(dataList)
                        else {
                            adapter?.setEmptyView(R.layout.layout_search_nothing, recyclerView)
                            val tvEmpty = adapter?.emptyView?.findViewById<TextView>(R.id.tv_empty_hint)
                            tvEmpty?.text = "你的口味太独特了，我们找不到"
                        }

                    }
                    SEARCH_MORE_TYPE -> {
                        refreshLayout.finishLoadMore()
                        val searchContentModel = responseModel.data as SearchContentModel
                        val dataList = searchContentModel.data
                        if (dataList.isNotEmpty())
                            adapter?.addData(dataList)
                        else
                            refreshLayout.finishLoadMoreWithNoMoreData()

                    }
                }
            }
            ON_FAILED -> {

                val responseModel = obj as ResponseFailModel

                if (responseModel.type == SEARCH_MORE_TYPE) {
                    pageIndex--
                    refreshLayout.finishLoadMore()

                } else if (responseModel.type == SEARCH_REFRESH_TYPE) {
                    refreshLayout.finishRefresh()
                    adapter?.setEmptyView(R.layout.layout_search_nothing, recyclerView)

                }

            }

        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (resultCode == DetailTodaySeenActivity.result_code_delete_content) {
            val deletePosition = data?.getIntExtra("position", 0) ?: 0
            adapter?.data?.removeAt(deletePosition)
            adapter?.notifyDataSetChanged()
        } else {
            if (data?.getSerializableExtra("likeModel") != null) {
                val likeModel = data.getSerializableExtra("likeModel") as LikeModel
                val searchContentModel = adapter?.getItem(likeModel.position)
                searchContentModel?.likeCount = (likeModel.likeCount)
                searchContentModel?.isLike = (likeModel.isLike)
                adapter?.setData(likeModel.position, searchContentModel!!)
            }
        }


    }

    fun setData(name: String?) {
        this.name = name
        pageIndex = 1
        getSearchData(pageIndex, SEARCH_REFRESH_TYPE)
    }

}
