package com.gfeng.daydaycook.search


import android.content.Intent
import android.os.Bundle
import android.support.v7.widget.GridLayoutManager
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import com.alibaba.fastjson.JSONObject
import com.alibaba.fastjson.TypeReference
import com.gfeng.daydaycook.R
import com.gfeng.daydaycook.activity.DetailsRecipeActivity
import com.gfeng.daydaycook.activity.DetailsThemeActivity
import com.gfeng.daydaycook.comm.Comm
import com.gfeng.daydaycook.comm.Global
import com.gfeng.daydaycook.fragment.BaseFragment
import com.gfeng.daydaycook.logic.AppMgr
import com.gfeng.daydaycook.model.*
import com.gfeng.daydaycook.search.adapter.SearchRecipeResultAdapter
import com.gfeng.daydaycook.search.entity.SearchContentModel
import com.gfeng.daydaycook.util.LogUtils
import com.scwang.smartrefresh.layout.api.RefreshLayout
import com.scwang.smartrefresh.layout.listener.OnRefreshLoadMoreListener
import kotlinx.android.synthetic.main.fragment_search_recipe.*
import net.lucode.hackware.magicindicator.buildins.UIUtil

/**
 * Life Is Better With DayDayCook
 * author: HuangQiang
 * created on: 2019/4/16 14:54.
 * description:搜索的老食谱列表页
 */
class SearchRecipeFragment : BaseFragment() {

    private var name: String? = ""//搜索词
    private var catalogId: Int? = 0//搜索分类id
    private var pageIndex = 1//分页页码 从1开始
    private val pageSize = 10

    private var flavorSelectId = -1
    private var skillSelectId = -1
    private var cookTimeSelectId = -1

    private var headView: View? = null
    private var searchNewModel: SearchNewModel? = null
    private var searchContentModel: SearchContentModel? = null

    private var tagDeleteListener: SearchResultActivity.TagDeleteListener? = null
    private var clickItemId = -1
    private var adapter: SearchRecipeResultAdapter? = null

    companion object {
        private const val TAG: String = "SearchRecipeFragment"
        private const val SEARCH_REFRESH_TYPE = 101
        private const val SEARCH_MORE_TYPE = 102
        private const val SEARCH_REFRESH_CONTENT_TYPE = 103
        const val favorite_add_refresh_type = 104
        const val favorite_del_refresh_type = 105
        const val add_favorite_refresh_type = 203
        const val del_favorite_refresh_type = 204

        fun getInstance(name: String): SearchRecipeFragment {
            val fragment = SearchRecipeFragment()
            val bundle = Bundle()
            bundle.putString("name", name)
            fragment.arguments = bundle
            return fragment
        }

        fun getInstance(name: String, catalogId: Int): SearchRecipeFragment {
            val fragment = SearchRecipeFragment()
            val bundle = Bundle()
            bundle.putString("name", name)
            bundle.putInt("catalogId", catalogId)
            fragment.arguments = bundle
            return fragment
        }
    }

    override fun getFragmentViewLayout(): Int {
        return R.layout.fragment_search_recipe
    }


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        name = arguments?.getString("name") ?: ""
        catalogId = arguments?.getInt("catalogId", 0) ?: 0

        Global.mAppMgr.setActivityDataRefreshListener(this, Comm.searchrecipefragment_refresh_type)

        LogUtils.e(TAG, "onCreate")
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initView()
    }

    private fun initView() {
        refreshLayout.setOnRefreshLoadMoreListener(object : OnRefreshLoadMoreListener {
            override fun onLoadMore(refreshLayout: RefreshLayout?) {
                LogUtils.e(TAG, "onLoadMore")
                getSearchData(++pageIndex, SEARCH_MORE_TYPE)
            }

            override fun onRefresh(refreshLayout: RefreshLayout?) {
                LogUtils.e(TAG, "onRefresh")
//                refreshSearchData()
                getSearchContentData(1, SEARCH_REFRESH_CONTENT_TYPE)
            }
        })

        refreshLayout.isEnableAutoLoadMore = true//滑动到底部自动加载更多
        refreshLayout.autoRefresh()//第一次加载数据

        val gridLayoutManager = object : GridLayoutManager(context, 2) {}
        recyclerView.layoutManager = gridLayoutManager

        adapter = SearchRecipeResultAdapter(R.layout.layout_item_recipe_all)
        recyclerView.adapter = adapter
        initHeadEmptyView()
        gridLayoutManager.spanSizeLookup = object : GridLayoutManager.SpanSizeLookup() {
            override fun getSpanSize(p0: Int): Int {
                return if (adapter?.data?.isNotEmpty() == true) {
                    if (p0 == 0 || adapter?.data!![p0 - 1].type == 2) {
                        2
                    } else {
                        1
                    }
                } else {
                    2
                }

            }
        }

        adapter?.setOnItemClickListener { adapter, _, position ->
            val model = adapter.data[position] as SearchAllModel
            if (model.type == 2) {
                Global.mAppMgr.refreshActivityData(Comm.search_recipe_position, Comm.search_recipe_position, 1)
            } else if (model.type == 1) {

            } else if (model.type == 0) {
                if (model.searchRecipeModel.type == "00") {
                    val intent = Intent(context, DetailsRecipeActivity::class.java)
                    intent.putExtra(DetailsRecipeActivity.ID, model.searchRecipeModel.recipeContentId)
                    startActivity(intent)
                } else if (model.searchRecipeModel.type == "01") {
                    val intent = Intent(context, DetailsThemeActivity::class.java)
                    intent.putExtra(DetailsThemeActivity.ID, model.searchRecipeModel.recipeContentId)
                    startActivity(intent)
                }
            }
        }
        adapter?.setOnItemChildClickListener { adapter, view, position ->
            when (view.id) {
                R.id.like -> {
                    if (TextUtils.isEmpty(Global.getUserUniqueId())) {
                        AppMgr.login(activity)

                    } else {
                        val model = adapter.data[position] as SearchAllModel
                        val favMap = JSONObject()
                        clickItemId = model.searchRecipeModel.recipeContentId
                        favMap["itemId"] = model.searchRecipeModel.recipeContentId
                        favMap["type"] = 1
                        favMap["session"] = Global.getUserUniqueId()
                        val favorite = model.searchRecipeModel.favorite
                        if (!favorite) {
                            httpSend(null, Global.mcnHost, Comm.getMcnURL(Comm.RECIPE_ADD_FAVORITE), favMap, add_favorite_refresh_type)
                        } else {
                            httpSend(null, Global.mcnHost, Comm.getMcnURL(Comm.RECIPE_DEL_FAVORITE), favMap, del_favorite_refresh_type)
                        }

                    }
                }
            }
        }

    }

    /**
     * 获取搜索结果
     * @param index Int
     */
    private fun getSearchData(index: Int, refreshType: Int) {
        LogUtils.e(TAG, "search recipe index=$index")
        searchNewModel = null
//        if (index == 1) {
//            getSearchContentData(1, SEARCH_REFRESH_CONTENT_TYPE)
//        }
        val searchMap = JSONObject()
        searchMap["currPage"] = index
        searchMap["pageSize"] = pageSize
        searchMap["session"] = Global.getUserUniqueId()
//        searchMap["notheme"] = "2"

        if (!TextUtils.isEmpty(name)) {
            searchMap["name"] = name
        }

        if (catalogId != 0) {
            searchMap["catalogId"] = catalogId
        }

        var screeningId = ""

        if (flavorSelectId != -1) {
            screeningId += ",$flavorSelectId"
        }

        if (skillSelectId != -1) {
            screeningId += ",$skillSelectId"
        }

        if (cookTimeSelectId != -1) {
            screeningId += ",$cookTimeSelectId"
        }

        if (!TextUtils.isEmpty(screeningId)) {
            searchMap["screeningId"] = screeningId.replaceFirst(",".toRegex(), "")
        }

        httpSend(object : TypeReference<SearchNewModel>() {}.type, Global.recipeSearch, Comm.RECIPE_SEARCH, searchMap, refreshType)

    }

    override fun aidHandleMessage(what: Int, obj: Any?) {
        super.aidHandleMessage(what, obj)
        when (what) {

            favorite_add_refresh_type -> {
                if (obj == null)
                    return

                val addId = Integer.parseInt(obj.toString())

                val addModel = SearchModel()
                addModel.recipeContentId = addId

                val searchAllModel = object : SearchAllModel(0, name, addModel, null) {}
                val index = adapter?.data?.indexOf(searchAllModel) ?: -1
                if (index >= 0 && adapter?.data != null && index < adapter?.data!!.size) {
                    val tempAddModel = adapter?.data?.get(index)?.searchRecipeModel
                    if (tempAddModel != null) {
                        tempAddModel.favoriteCount = tempAddModel.favoriteCount + 1
                    }
                    tempAddModel?.favorite = true

                    adapter?.notifyItemChanged(index + 1)
                }
            }
            favorite_del_refresh_type -> {
                if (obj == null)
                    return

                val delId = Integer.parseInt(obj.toString())
                val delModel = SearchModel()
                delModel.recipeContentId = delId

                val searchAllModel = object : SearchAllModel(0, name, delModel, null) {}
                val index = adapter?.data?.indexOf(searchAllModel) ?: -1

                if (index >= 0 && adapter?.data != null && index < adapter?.data!!.size) {
                    val tempDelModel = adapter?.data?.get(index)?.searchRecipeModel
                    if (tempDelModel != null) {
                        tempDelModel.favoriteCount = tempDelModel.favoriteCount - 1
                    }
                    tempDelModel?.favorite = false

                    adapter?.notifyItemChanged(index + 1)
                }
            }
            ON_SUCCESS -> {
                hideProgressDialog()

                val responseModel = obj as ResponseModel
                if (responseModel.code != Comm.CODE_200) {
                    if (responseModel.type == SEARCH_MORE_TYPE) {
                        refreshLayout.finishLoadMoreWithNoMoreData()
                    } else if (responseModel.type == SEARCH_REFRESH_TYPE) {
                        refreshNoData()
                    }
                    return
                }

                when (responseModel.type) {
                    add_favorite_refresh_type -> {
                        LogUtils.i("favorite", "clickItemId = $clickItemId")
                        if (clickItemId == -1) return
                        val model = SearchAllModel()
                        model.searchRecipeModel.recipeContentId = clickItemId
                        val index = adapter?.data?.indexOf(model) ?: -1

                        LogUtils.i("favorite", "addIndex = $index")
                        if (index >= 0 && adapter?.data != null && index < adapter?.data!!.size) {
                            val tempAddModel = adapter?.data?.get(index)?.searchRecipeModel
                            if (tempAddModel != null) {
                                tempAddModel.favoriteCount = tempAddModel.favoriteCount + 1
                            }
                            tempAddModel?.favorite = true
                            adapter?.notifyItemChanged(index + 1)
                        }
                        clickItemId = -1
                    }
                    del_favorite_refresh_type -> {
                        LogUtils.i("favorite", "clickItemId = $clickItemId")
                        if (clickItemId == -1) return
                        val delModel = SearchAllModel()
                        delModel.searchRecipeModel.recipeContentId = clickItemId
                        val index = adapter?.data?.indexOf(delModel) ?: -1
                        LogUtils.i("favorite", "delIndex = $index")
                        if (index >= 0 && adapter?.data != null && index < adapter?.data!!.size) {
                            val tempDelModel = adapter?.data?.get(index)?.searchRecipeModel
                            if (tempDelModel != null) {
                                tempDelModel.favoriteCount = tempDelModel.favoriteCount - 1
                            }
                            tempDelModel?.favorite = false
                            adapter?.notifyItemChanged(index + 1)
                        }
                        clickItemId = -1
                    }
                    SEARCH_REFRESH_TYPE -> {
                        refreshLayout.finishRefresh()
                        searchNewModel = responseModel.data as SearchNewModel
                        if (TextUtils.equals(searchNewModel?.isRecommend, "1")) {
                            ll_head?.visibility = View.VISIBLE
                            //推荐只有10条无加载更多
                            refreshLayout.finishLoadMoreWithNoMoreData()
                        } else {
                            ll_head?.visibility = View.GONE
//                            ll_head?.visibility = View.GONE
                        }
                        if (searchNewModel?.results!!.isNotEmpty()) {
                            //设置数据
                            setNewData(searchNewModel?.results!!)
                        } else
                            refreshNoData()
                    }
                    SEARCH_REFRESH_CONTENT_TYPE -> {
//                        refreshLayout.finishRefresh()
                        searchContentModel = responseModel.data as SearchContentModel
                        val dataList = searchContentModel?.data
                        if (dataList == null || dataList.isEmpty()) {
                            btn_go_publish_head?.visibility = View.GONE
                        } else {
                            btn_go_publish_head?.visibility = View.VISIBLE
                        }

                        refreshSearchData()

                    }
                    SEARCH_MORE_TYPE -> {
                        refreshLayout.finishLoadMore()

                        val searchNewModel = responseModel.data as SearchNewModel
                        if (searchNewModel.results.isNotEmpty()) {
                            setAddData(searchNewModel.results)
//                            adapter?.addData(searchNewModel.results)
                        } else
                            refreshLayout.finishLoadMoreWithNoMoreData()
                    }
                }
            }
            ON_FAILED -> {
                val responseModel = obj as ResponseFailModel
                hideProgressDialog()

                when {
                    responseModel.type == SEARCH_MORE_TYPE -> {
                        pageIndex--
                        refreshLayout.finishLoadMore()
                    }
                    responseModel.type == SEARCH_REFRESH_TYPE -> refreshNoData()
                    responseModel.type == SEARCH_REFRESH_CONTENT_TYPE -> {
                        btn_go_publish_head?.visibility = View.GONE
                        refreshSearchData()
                    }
                }
            }
        }
    }

    /**
     * 设置数据
     */
    fun setNewData(list: List<SearchModel>) {
        val listNew: ArrayList<SearchAllModel> = ArrayList()
        if (list.isNotEmpty()) {
            for (i in 0 until list.size) {
                val searchAllModel = object : SearchAllModel(0, name, list[i], null) {}
                listNew.add(searchAllModel)
            }
            if (searchContentModel?.data != null && searchContentModel?.data!!.size >= 3) {
                val searchAllModel = object : SearchAllModel(2, name, null, searchContentModel?.data!!.subList(0, 3)) {}
                listNew.add(searchAllModel)
            }
        }
        LogUtils.i("setData", "setNewData==" + listNew.size)
        adapter?.setNewData(listNew)
    }

    /**
     * 设置加载更多的数据
     */
    private fun setAddData(list: List<SearchModel>) {
        val listNew: ArrayList<SearchAllModel> = ArrayList()
        if (list.isNotEmpty()) {
            for (i in 0 until list.size) {
                val searchAllModel = object : SearchAllModel(0, name, list[i], null) {}
                listNew.add(searchAllModel)
            }
            val index: Int = adapter?.data?.size!! / 11
            if (searchContentModel?.data != null && searchContentModel?.data!!.size >= 3 * (index + 1)) {
                val searchAllModel = object : SearchAllModel(2, name, null, searchContentModel?.data!!.subList(index * 3, (index + 1) * 3)) {}
                listNew.add(searchAllModel)
            }
        }
        LogUtils.i("setData", "setAddData==" + listNew.size)
        adapter?.addData(listNew)
    }

    /**
     * 没有搜索内容
     */
    private fun refreshNoData() {
        if (refreshLayout != null)
            refreshLayout?.finishRefresh()
        adapter?.setNewData(ArrayList())
        adapter?.setEmptyView(R.layout.layout_search_nothing, recyclerView)
        val tvEmpty = adapter?.emptyView?.findViewById<TextView>(R.id.tv_empty_hint)
        tvEmpty?.text = "你的口味太独特了，我们找不到"
    }


    /**
     * 搜索条件过滤
     * @param favorModel ScreeningNextModel?
     * @param skillModel ScreeningNextModel?
     * @param timeModel ScreeningNextModel?
     * @param tagDeleteListener SearchResultActivity.TagDeleteListener
     */
    fun setSelectCondition(favorModel: ScreeningNextModel?, skillModel: ScreeningNextModel?, timeModel: ScreeningNextModel?, tagDeleteListener: SearchResultActivity.TagDeleteListener) {

        flavorSelectId = favorModel?.technologyId ?: -1
        skillSelectId = skillModel?.technologyId ?: -1
        cookTimeSelectId = timeModel?.technologyId ?: -1

        tagLayout.removeAllViews()
        addTagView(favorModel, 0)
        addTagView(skillModel, 1)
        addTagView(timeModel, 2)

        showProgressDialog()
        refreshSearchData()

        this.tagDeleteListener = tagDeleteListener
        setRecyclerViewPadding()
    }

    fun setData(name: String?, catalogId: Int?) {
        this.name = name
        this.catalogId = catalogId

//        refreshSearchData()
        getSearchContentData(1, SEARCH_REFRESH_CONTENT_TYPE)
    }

    private fun refreshSearchData() {
        if (refreshLayout != null)
            refreshLayout?.setNoMoreData(false)//重新可以加载更多
        pageIndex = 1
        getSearchData(pageIndex, SEARCH_REFRESH_TYPE)
    }

    private fun addTagView(screeningNextModel: ScreeningNextModel?, type: Int) {
        if (screeningNextModel != null) {
            val view = LayoutInflater.from(context).inflate(R.layout.layout_search_filter_item, null)
            val nameTv = view.findViewById(R.id.nameTv) as TextView
            val deleteIv = view.findViewById(R.id.deleteIv) as ImageView
            nameTv.text = screeningNextModel.name
            deleteIv.setOnClickListener {
                tagLayout.removeView(view)
                when (type) {
                    0 -> flavorSelectId = -1
                    1 -> skillSelectId = -1
                    2 -> cookTimeSelectId = -1
                }

                refreshSearchData()
                tagDeleteListener?.onTagDelete(type)
                setRecyclerViewPadding()
            }
            tagLayout.addView(view)
            setRecyclerViewPadding()
        }
    }

    private fun setRecyclerViewPadding() {
        val dip7 = UIUtil.dip2px(context, 7.00)
        if (tagLayout.childCount > 0) {
            recyclerView.setPadding(0, 0, 0, 0)
        } else {
            recyclerView.setPadding(0, dip7 * 2, 0, 0)
            recyclerView.scrollBy(0, recyclerView.scrollY - dip7 * 2)
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()

        Global.mAppMgr.setActivityDataRefreshListener(null, Comm.searchrecipefragment_refresh_type)

    }

    /**
     * 获取搜索结果
     * @param index Int
     */
    private fun getSearchContentData(index: Int, refreshType: Int) {
        if (refreshLayout != null)
            refreshLayout?.setNoMoreData(false)//重新可以加载更多
        LogUtils.e(TAG, "search content index=$index")

        val jsonObject = JSONObject()
        jsonObject["q"] = name
        jsonObject["pageIndex"] = index
        jsonObject["pageRow"] = 20
        jsonObject["sessionId"] = Global.getUserUniqueId()
        httpPost(object : TypeReference<SearchContentModel>() {}.type, Global.tvHost, Comm.contentSearch, jsonObject, refreshType)
    }

    private var ll_head: View? = null
    private var btn_go_publish_head: Button? = null
    private var tv_content_head: TextView? = null
    /**
     * 设置顶部面状态
     */
    private fun initHeadEmptyView() {
        headView = LayoutInflater.from(context).inflate(R.layout.layout_head_search, null)
        adapter!!.setHeaderView(headView)
        ll_head = headView?.findViewById(R.id.ll_head)
        btn_go_publish_head = headView?.findViewById(R.id.btn_go_publish_head)
        tv_content_head = headView?.findViewById(R.id.tv_content_head)
        ll_head?.visibility = View.GONE
        btn_go_publish_head?.text = "去社区逛逛"
        tv_content_head?.text = "你的口味太独特了，我们找不到\n" + "给您推荐其他精彩内容"
        btn_go_publish_head?.setOnClickListener {
            Global.mAppMgr.refreshActivityData(Comm.search_recipe_position, Comm.search_recipe_position, 1)
        }
    }

}
