package com.gfeng.daydaycook.search

import android.content.Context
import android.content.Intent
import android.graphics.Color
import android.os.Bundle
import android.support.v4.app.Fragment
import android.support.v4.content.ContextCompat
import android.support.v4.view.ViewPager
import android.support.v7.widget.LinearLayoutManager
import android.text.InputFilter
import android.text.TextUtils
import android.view.View
import android.view.inputmethod.EditorInfo
import com.alibaba.fastjson.JSONObject
import com.alibaba.fastjson.TypeReference
import com.gfeng.daydaycook.R
import com.gfeng.daydaycook.activity.BaseActivity
import com.gfeng.daydaycook.comm.Comm
import com.gfeng.daydaycook.comm.Global
import com.gfeng.daydaycook.discover.widget.TitleView
import com.gfeng.daydaycook.model.PopularSearchWordsModel
import com.gfeng.daydaycook.model.ResponseFailModel
import com.gfeng.daydaycook.model.ResponseModel
import com.gfeng.daydaycook.model.ScreeningNextModel
import com.gfeng.daydaycook.search.adapter.HistoryTagAdapter
import com.gfeng.daydaycook.search.adapter.HotSearchTagAdapter
import com.gfeng.daydaycook.search.adapter.SearchMindAdapter
import com.gfeng.daydaycook.search.adapter.SearchResultPagerAdapter
import com.gfeng.daydaycook.search.entity.HistoryModel
import com.gfeng.daydaycook.search.entity.SearchMindModel
import com.gfeng.daydaycook.util.KeyboardUtils
import com.gfeng.daydaycook.util.LogUtils
import com.gfeng.daydaycook.util.Utils
import com.gfeng.daydaycook.widget.EditTextWithDel
import com.google.gson.reflect.TypeToken
import kotlinx.android.synthetic.main.activity_search_result.*
import net.lucode.hackware.magicindicator.ViewPagerHelper
import net.lucode.hackware.magicindicator.buildins.UIUtil
import net.lucode.hackware.magicindicator.buildins.commonnavigator.CommonNavigator
import net.lucode.hackware.magicindicator.buildins.commonnavigator.abs.CommonNavigatorAdapter
import net.lucode.hackware.magicindicator.buildins.commonnavigator.abs.IPagerIndicator
import net.lucode.hackware.magicindicator.buildins.commonnavigator.abs.IPagerTitleView
import net.lucode.hackware.magicindicator.buildins.commonnavigator.indicators.LinePagerIndicator
import java.util.*
import java.util.regex.Pattern
import kotlin.collections.ArrayList

/**
 * Life Is Better With DayDayCook
 * author: HuangQiang
 * created on: 2019/4/15 14:30.
 * description:搜索
 */
class SearchResultActivity : BaseActivity() {

    private var name: String = ""//搜索的文字
    private var catalogId: Int = 0//搜索的分类

    private var mTitles: Array<String>? = null
    private var fragmentArray: Array<Fragment>? = null
    private var favorModel: ScreeningNextModel? = null
    private var skillModel: ScreeningNextModel? = null
    private var timeModel: ScreeningNextModel? = null
    private var searchRecipeFragment: SearchRecipeFragment? = null
    private var searchContentFragment: SearchContentFragment? = null
    private var searchUserFragment: SearchUserFragment? = null
    private var historyAdapter: HistoryTagAdapter? = null


    companion object {
        private val TAG = SearchResultActivity::class.java.simpleName
        private const val REQUEST_CODE_FILTER = 11
        private const val search_hot_refresh_type = 101
        private const val SEARCH_REFRESH_TYPE = 102
        private const val SEARCH_AIDL = 103

        fun goToPage(context: Context) {
            val intent = Intent(context, SearchResultActivity::class.java)
            context.startActivity(intent)
        }

        fun goToPage(context: Context, name: String, catagoryId: Int) {
            val intent = Intent(context, SearchResultActivity::class.java)
            intent.putExtra("name", name)
            intent.putExtra("catalogId", catagoryId)
            context.startActivity(intent)
        }
    }


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_search_result)
        if (intent.hasExtra("name"))
            name = intent.getStringExtra("name")
        if (intent.hasExtra("catalogId"))
            catalogId = intent.getIntExtra("catalogId", 0)

        initView()
        Global.mAppMgr.setActivityDataRefreshListener(this, Comm.search_recipe_position)
    }

    private fun initView() {

        cancelTv.setOnClickListener {
            onBackPressed()
        }

        initSearchEdit()
        initHistorySearch()
        initHotSearch()

        if (catalogId > 0) {
            startSearch()
        }

        filterIv.setOnClickListener {
            FilterActivity.goToPage(this, favorModel, skillModel, timeModel, REQUEST_CODE_FILTER)
        }
        initMindView()
    }

    var task:TimerTask?=null
    val timer = Timer()
    var inputTime = System.currentTimeMillis()
    /**
     * 初始化搜索框
     */
    private fun initSearchEdit() {
        searchEt.requestFocus()
        searchEt.setOnClickListener {
            LogUtils.e(TAG, "searchEt onClick")
            searchEt.isCursorVisible = true
            if(recyclerView.visibility==View.GONE){
            searchLayout.visibility = View.VISIBLE
            resultLayout.visibility = View.GONE
            recyclerView.visibility = View.GONE
            }
        }

        searchEt.setOnEditorActionListener { _, actionId, _ ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH) {
                val editText = searchEt.text.toString()
                if (editText.isNotEmpty()) {
                    name = editText
                    startSearch()
                } else {
                    val hintStr = searchEt.hint.toString()
                    if (hintStr.isNotEmpty() && hintStr != getString(R.string.search_content_prompt)) {
                        name = hintStr
                        catalogId = 0
                        startSearch()
                    }
                }
            }
            true
        }
        val filter = InputFilter { source, _, _, _, _, _ ->
            if (source == " ")
            //空格
                return@InputFilter ""
            val speChat = "[`~!@#$%^&*()+=|{}':;',\\[\\].<>/?~！@#￥%……&*（）——+|{}【】‘；：”“’。，、？]"
            val pattern = Pattern.compile(speChat)
            val matcher = pattern.matcher(source.toString())
            if (matcher.find())
                ""
            else
                null
        }
        //禁止EditText输入特殊字符
        searchEt.filters = arrayOf(filter)

        searchEt.mListener = object : EditTextWithDel.InputListener{
            override fun afterInput(str: String) {
                LogUtils.i("afterInput", "afterInput==$str")
                catalogId = 0
                inputTime = System.currentTimeMillis()
                task = object : TimerTask() {
                    override fun run() {
                        if (System.currentTimeMillis()-inputTime<500){
                            //说明已经有新的搜索，
                            LogUtils.i("afterInput","被替换=="+str)
                        }else{
                            LogUtils.i("afterInput","无替换=="+str)
                            //todo 网络请求
                            aidsendMessage(SEARCH_AIDL, str)
                        }
                        cancel()
                    }
                }
                timer.schedule(task,500)

                if(TextUtils.isEmpty(str)){
                    searchLayout.visibility = View.VISIBLE
                    recyclerView.visibility = View.GONE
                }
            }
        }

    }

    var adapter:SearchMindAdapter?=null
    private fun initMindView(){
        recyclerView.layoutManager = LinearLayoutManager(this)
        adapter = SearchMindAdapter(R.layout.item_search_mind)
        recyclerView.adapter = adapter
        adapter?.setOnItemClickListener{ adapter, _, position ->
            val model = adapter.data[position] as SearchMindModel.Data
            if (!TextUtils.isEmpty(model.name)) {
                name = model.name
                startSearch()
            }
        }
    }


    /**
     * 历史搜索
     */
    private fun initHistorySearch() {
        val historyStr = Utils.getLocalSave(Global.mContext, Comm.SAVEFILE, Comm.SEARCH_HISTORY)
        var historyModelList = historyStr.split(",".toRegex()).dropLastWhile { it.isEmpty() }.map { HistoryModel(it) }


        if (historyModelList.isEmpty()) {
            historyRl.visibility = View.GONE
            historyFlow.visibility = View.GONE
            return
        } else {
            if (historyModelList.size > 10) {
                //原来历史记录多余10个只保留10个
                val sb = StringBuilder(historyStr)
                for (i in 0..9) {
                    sb.insert(0, historyModelList[i].string + ",")
                }
                Utils.setLocalSave(Global.mContext, Comm.SAVEFILE, Comm.SEARCH_HISTORY, sb.toString())
                historyModelList = historyModelList.subList(0, 9)
            }

            historyRl.visibility = View.VISIBLE
            historyFlow.visibility = View.VISIBLE
        }

        searchEt.hint = historyModelList[0].string

        historyAdapter = HistoryTagAdapter(this, historyModelList)
        historyFlow.adapter = historyAdapter
        historyFlow.setOnTagClickListener { _, position, _ ->
            name = historyAdapter?.getItem(position)?.string ?: ""
            startSearch()
            false
        }

        clearIv.setOnClickListener {
            Utils.setLocalSave(Global.mContext, Comm.SAVEFILE, Comm.SEARCH_HISTORY, "")
            initHistorySearch()
        }

    }

    /**
     * 保存历史搜索记录,最多保留10个
     */
    private fun saveHistory() {
        val text = searchEt.text.toString()
        var historyStr = Utils.getLocalSave(Global.mContext, Comm.SAVEFILE, Comm.SEARCH_HISTORY)
        val historyList = historyStr.split(",".toRegex()).dropLastWhile { it.isEmpty() }

        if (!historyStr.contains("$text,")) {
            if (historyList.size > 9)
                historyStr = historyStr.replace(historyList[historyList.size - 1] + ",", "")//清除第10个的

        } else {
            historyStr = historyStr.replace("$text,", "")//清除原有的
        }

        val sb = StringBuilder(historyStr)
        sb.insert(0, "$text,")
        Utils.setLocalSave(Global.mContext, Comm.SAVEFILE, Comm.SEARCH_HISTORY, sb.toString())
    }


    /**
     * 热门搜索
     */
    private fun initHotSearch() {
        sendHttp(object : TypeToken<List<PopularSearchWordsModel>>() {}.type, Comm.hotSearch, null, search_hot_refresh_type)
        hotFlow.setOnTagClickListener { _, position, _ ->
            name = hotAdapter?.getItem(position)?.htagname ?: ""
            startSearch()
            false
        }
    }

    /**
     * 开始搜索
     */
    private fun startSearch() {
        LogUtils.e(TAG, "startSearch")

        searchEt.setText(name)
        searchEt.isCursorVisible = false
        KeyboardUtils.hideSoftInput(this)
        initViewPager()
        searchLayout.visibility = View.GONE
        resultLayout.visibility = View.VISIBLE
        recyclerView.visibility = View.GONE

        saveHistory()
        initHistorySearch()

    }


    private var commonNavigator:CommonNavigator ?=null
    /**
     * 初始化tabs
     */
    private fun initMagicIndicator() {
        commonNavigator = CommonNavigator(this)
        val commonNavigatorAdapter = object : CommonNavigatorAdapter() {
            override fun getCount(): Int {
                return mTitles?.size ?: 0
            }

            override fun getTitleView(context: Context, index: Int): IPagerTitleView {
                val titleView = TitleView(context)

                titleView.text = mTitles?.get(index) ?: ""
                titleView.textSize = 16f

                titleView.normalColor = Color.parseColor("#000000")
                titleView.selectedColor = Color.parseColor("#1A1A1A")

                titleView.setOnClickListener {
                    viewpager.currentItem = index
                }

                return titleView
            }

            override fun getIndicator(context: Context): IPagerIndicator {
                val indicator = LinePagerIndicator(context)
                indicator.setColors(ContextCompat.getColor(context, R.color.main_style_color))
                indicator.mode = LinePagerIndicator.MODE_EXACTLY
                indicator.lineHeight = UIUtil.dip2px(context, 4.0).toFloat()
                indicator.lineWidth = UIUtil.dip2px(context, 20.0).toFloat()
                return indicator
            }
        }
        commonNavigator?.adapter = commonNavigatorAdapter
        indicator.navigator = commonNavigator
        ViewPagerHelper.bind(indicator, viewpager)
    }

    private fun setTabIndex(position:Int){
        viewpager.currentItem = position
    }

    private fun initViewPager() {


        if (searchRecipeFragment == null)
            searchRecipeFragment = SearchRecipeFragment.getInstance(name, catalogId)
        else
            searchRecipeFragment!!.setData(name, catalogId)

        if (searchContentFragment == null)
            searchContentFragment = SearchContentFragment.getInstance(name)
        else
            searchContentFragment!!.setData(name)

        if (searchUserFragment == null)
            searchUserFragment = SearchUserFragment.getInstance(name)
        else
            searchUserFragment!!.setData(name)

        fragmentArray = arrayOf(searchRecipeFragment!!, searchContentFragment!!, searchUserFragment!!)

        mTitles = arrayOf("食谱", "社区", "用户")

        viewpager.offscreenPageLimit = 3



        initMagicIndicator()

        viewpager.adapter = SearchResultPagerAdapter(supportFragmentManager, fragmentArray!!, mTitles!!)
        viewpager.addOnPageChangeListener(object : ViewPager.OnPageChangeListener {
            override fun onPageScrolled(position: Int, positionOffset: Float, positionOffsetPixels: Int) {

            }

            override fun onPageSelected(position: Int) {
                if (position == 0)
                    filterIv.visibility = View.VISIBLE
                else
                    filterIv.visibility = View.GONE

            }

            override fun onPageScrollStateChanged(state: Int) {

            }
        })

        viewpager.currentItem = 0
        filterIv.visibility = View.VISIBLE

    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == REQUEST_CODE_FILTER && resultCode == RESULT_OK) {
            if (data?.hasExtra(FilterActivity.flavorSelectName) == true)
                favorModel = data.getSerializableExtra(FilterActivity.flavorSelectName) as ScreeningNextModel
            if (data?.hasExtra(FilterActivity.skillSelectName) == true)
                skillModel = data.getSerializableExtra(FilterActivity.skillSelectName) as ScreeningNextModel
            if (data?.hasExtra(FilterActivity.timeSelectName) == true)
                timeModel = data.getSerializableExtra(FilterActivity.timeSelectName) as ScreeningNextModel

            searchRecipeFragment?.setSelectCondition(favorModel, skillModel, timeModel, tagDeleteListener)
        }
    }

    private var tagDeleteListener = object : TagDeleteListener() {
        override fun onTagDelete(type: Int) {
            when (type) {
                0 -> favorModel = null
                1 -> skillModel = null
                2 -> timeModel = null
            }
        }

    }

    abstract class TagDeleteListener {
        abstract fun onTagDelete(type: Int)
    }

    private var hotAdapter: HotSearchTagAdapter? = null
    private var searchMindModel: SearchMindModel? = null

    override fun aidHandleMessage(what: Int, obj: Any?) {
        super.aidHandleMessage(what, obj)
        when (what) {
            Comm.search_recipe_position ->{
                val position = obj as Int
                setTabIndex(position)
            }
            SEARCH_AIDL ->{
                val str = obj as String
                getSearchMindData(str)
            }
            ON_SUCCESS -> {
                val responseModel = obj as ResponseModel
                when (responseModel.type) {
                    SEARCH_REFRESH_TYPE -> {
//                        refreshLayout.finishRefresh()
                        searchMindModel = responseModel.data as SearchMindModel
                        var dataList = searchMindModel?.data
                        if (dataList == null || dataList.isEmpty()) {
//                            recyclerView?.visibility = View.GONE
                            dataList = ArrayList()
                        } else {
                        }
                        if(resultLayout.visibility==View.GONE)
                        {
                            if(TextUtils.isEmpty(searchEt.text)){
                                recyclerView?.visibility = View.GONE
                                searchLayout.visibility = View.VISIBLE
                            }else{
                                recyclerView?.visibility = View.VISIBLE
                                searchLayout.visibility = View.GONE
                            }
                        }
                        adapter?.replaceData(dataList)
                        LogUtils.i("responseModel","ON_SUCCESS")
                    }
                    search_hot_refresh_type -> {
                        val list = responseModel.data as List<PopularSearchWordsModel>
                        if (list.isNotEmpty()) {
                            hotAdapter = HotSearchTagAdapter(this, list)
                            hotFlow.adapter = hotAdapter
                        }
                    }
                }
            }
            ON_FAILED -> {
                val responseModel = obj as ResponseFailModel
                when (responseModel.type) {
                    SEARCH_REFRESH_TYPE -> {
                        LogUtils.i("responseModel","ON_FAILED")
                        adapter?.replaceData(ArrayList())
                    }
                }
            }
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        Global.mAppMgr.setActivityDataRefreshListener(null, Comm.search_recipe_position)
    }

    /**
     * 获取搜索结果
     */
    private fun getSearchMindData(str:String) {
        if(TextUtils.isEmpty(str))return

        val jsonObject = JSONObject()
        jsonObject["name"] = str
        httpPost(object : TypeReference<SearchMindModel>() {}.type, Global.recipeSearch, Comm.MIND_SEARCH, jsonObject, SEARCH_REFRESH_TYPE)
    }

}