package com.gfeng.daydaycook.search


import android.os.Bundle
import android.support.v7.widget.LinearLayoutManager
import android.text.TextUtils
import android.view.View
import android.widget.TextView
import com.alibaba.fastjson.JSONObject
import com.alibaba.fastjson.TypeReference
import com.gfeng.daydaycook.R
import com.gfeng.daydaycook.comm.Comm
import com.gfeng.daydaycook.comm.Global
import com.gfeng.daydaycook.fragment.BaseFragment
import com.gfeng.daydaycook.label.PersonalHomepageActivity
import com.gfeng.daydaycook.logic.AppMgr
import com.gfeng.daydaycook.model.ResponseFailModel
import com.gfeng.daydaycook.model.ResponseModel
import com.gfeng.daydaycook.model.choice.FocusModel
import com.gfeng.daydaycook.search.adapter.SearchUserResultAdapter
import com.gfeng.daydaycook.search.entity.SearchUserModel
import com.gfeng.daydaycook.util.LogUtils
import com.google.gson.reflect.TypeToken
import com.scwang.smartrefresh.layout.api.RefreshLayout
import com.scwang.smartrefresh.layout.listener.OnRefreshLoadMoreListener
import kotlinx.android.synthetic.main.fragment_search_recipe.*

/**
 * Life Is Better With DayDayCook
 * author: HuangQiang
 * created on: 2019/4/16 14:54.
 * description:搜索的用户列表页
 */
class SearchUserFragment : BaseFragment() {
    private val TAG: String = "SearchUserFragment"

    private var name: String? = ""//搜索词
    private var pageIndex = 1//分页页码 从0开始

    private val SEARCH_REFRESH_TYPE = 101
    private val SEARCH_MORE_TYPE = 102
    private val network_focus = 103


    companion object {
        fun getInstance(name: String): SearchUserFragment {
            val fragment = SearchUserFragment()
            val bundle = Bundle()
            bundle.putString("name", name)
            fragment.arguments = bundle
            return fragment
        }
    }

    override fun getFragmentViewLayout(): Int {
        return R.layout.fragment_search_user
    }


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        name = arguments?.getString("name") ?: ""
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initView(view)
    }

    private var adapter: SearchUserResultAdapter? = null

    private fun initView(view: View) {
        refreshLayout.setOnRefreshLoadMoreListener(object : OnRefreshLoadMoreListener {
            override fun onLoadMore(refreshLayout: RefreshLayout?) {
                LogUtils.e(TAG, "onLoadMore")
                getSearchData(++pageIndex, SEARCH_MORE_TYPE)
            }

            override fun onRefresh(refreshLayout: RefreshLayout?) {
                LogUtils.e(TAG, "onRefresh")
                refreshLayout?.setNoMoreData(false)//重新可以加载更多
                pageIndex = 1
                getSearchData(pageIndex, SEARCH_REFRESH_TYPE)
            }
        })

        refreshLayout.isEnableAutoLoadMore = true//滑动到底部自动加载更多
        refreshLayout.autoRefresh()//第一次加载数据

        recyclerView.layoutManager = LinearLayoutManager(context)
        adapter = SearchUserResultAdapter(R.layout.item_focus)
        recyclerView.adapter = adapter

        adapter?.setOnItemClickListener { adapter, _, position ->
            val model: SearchUserModel.Data = adapter.data[position] as SearchUserModel.Data
            PersonalHomepageActivity.startActivity(context, model.userInfo.userId)
        }
        adapter?.setOnItemChildClickListener { adapter, view, position ->
            when (view.id) {
                R.id.iv_focus_status -> {
                    if (TextUtils.isEmpty(Global.getUserUniqueId())) {
                        AppMgr.login(activity)

                    } else {
                        val model: SearchUserModel.Data = adapter.data[position] as SearchUserModel.Data
                        if (model.watchUser) {
                            focus(2, model.userInfo.userId)//取消关注
                        } else {
                            focus(1, model.userInfo.userId)//关注
                        }
                        model.watchUser = !model.watchUser
                        adapter.notifyItemChanged(position)
                    }
                }
            }
        }
    }

    /**
     * 获取搜索结果
     * @param index Int
     */
    private fun getSearchData(index: Int, refreshType: Int) {
        LogUtils.e(TAG, "search User index=$index")

        val jsonObject = JSONObject()
        jsonObject["name"] = name
        jsonObject["pageIndex"] = index
        jsonObject["pageRow"] = 20
        jsonObject["sessionId"] = Global.getUserUniqueId()
        httpPost(object : TypeReference<SearchUserModel>() {}.type, Global.tvHost, Comm.contentSearchUser, jsonObject, refreshType)
    }

    /**
     * 关注
     */
    private fun focus(type: Int, uid: String) {
        val jsonObject = JSONObject()
        jsonObject["sessionId"] = Global.getUserUniqueId()
        jsonObject["type"] = type
        jsonObject["userId"] = uid
        httpPost(object : TypeToken<FocusModel>() {}.type, Global.tvHost, Comm.FOCUS_USER, jsonObject, network_focus)
    }

    override fun aidHandleMessage(what: Int, obj: Any?) {
        super.aidHandleMessage(what, obj)
        when (what) {
            ON_SUCCESS -> {

                val responseModel = obj as ResponseModel

                when (responseModel.type) {
                    SEARCH_REFRESH_TYPE -> {
                        refreshLayout.finishRefresh()

                        val searchUserModel = responseModel.data as SearchUserModel
                        val dataList = searchUserModel.data
                        if (dataList.isNotEmpty())
                            adapter?.setNewData(dataList)
                        else {
                            adapter?.setEmptyView(R.layout.layout_search_nothing, recyclerView)
                            val tvEmpty = adapter?.emptyView?.findViewById<TextView>(R.id.tv_empty_hint)
                            tvEmpty?.text = "没有相关用户"
                        }
                    }
                    SEARCH_MORE_TYPE -> {
                        refreshLayout.finishLoadMore()
                        val searchUserModel = responseModel.data as SearchUserModel
                        val dataList = searchUserModel.data
                        if (dataList.isNotEmpty())
                            adapter?.addData(dataList)
                        else
                            refreshLayout.finishLoadMoreWithNoMoreData()
                    }
                }
            }
            ON_FAILED -> {

                val responseModel = obj as ResponseFailModel

                if (responseModel.type == SEARCH_MORE_TYPE) {
                    pageIndex--
                    refreshLayout.finishLoadMore()

                } else if (responseModel.type == SEARCH_REFRESH_TYPE) {
                    refreshLayout.finishRefresh()
                    adapter?.setEmptyView(R.layout.layout_search_nothing, recyclerView)

                }

            }

        }
    }

    fun setData(name: String?) {
        this.name = name
        pageIndex = 1
        getSearchData(pageIndex, SEARCH_REFRESH_TYPE)
    }
}
