package com.gfeng.daydaycook.search.adapter

import android.graphics.Color
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.BaseViewHolder
import com.gfeng.daydaycook.R
import com.gfeng.daydaycook.search.entity.SearchContentModel
import com.gfeng.daydaycook.util.GlideUtils
import com.gfeng.daydaycook.util.NumFormatUtil
import com.gfeng.daydaycook.util.ScreenUtil
import com.gfeng.daydaycook.util.TypefaceUtil
import com.makeramen.roundedimageview.RoundedImageView
import net.lucode.hackware.magicindicator.buildins.UIUtil

/**
 * Life Is Better With DayDayCook
 * author: HuangQiang
 * created on: 2019/4/16 17:28.
 * description:
 */
class SearchContentResultAdapter(layoutResId: Int) : BaseQuickAdapter<SearchContentModel.Data, BaseViewHolder>(layoutResId) {

    override fun convert(helper: BaseViewHolder, item: SearchContentModel.Data) {

        val item_bg = helper.getView<ImageView>(R.id.item_bg)
        val tv_deleted = helper.getView<TextView>(R.id.tv_deleted)
        val iv_headicon = helper.getView<RoundedImageView>(R.id.iv_headicon)
        val tv_count = helper.getView<TextView>(R.id.tv_count)
        val account_tv = helper.getView<TextView>(R.id.account_tv)
        val rv_time = helper.getView<View>(R.id.rv_time)
        val time_tv = helper.getView<TextView>(R.id.time_tv)
        val iv_thumbs_up = helper.getView<ImageView>(R.id.iv_thumbs_up)
        val tv_biankanbianmai = helper.getView<View>(R.id.tv_biankanbianmai)

        helper.setText(R.id.title, item.title)

        var url = item.smallPic
        if (url.isEmpty()) {
            url = item.coverImage
        }
        GlideUtils.loadCenterCrop(url, item_bg, R.drawable.default_ddc)
        GlideUtils.loadCircleCrop(item.header, R.drawable.default_user, iv_headicon)

        if (item.nickName.isEmpty()) {
            account_tv.text = "日日煮官方账号"
        } else {
            account_tv.text = item.nickName
        }

        tv_count.typeface = TypefaceUtil.getTypeface(mContext)
        tv_count.text = NumFormatUtil.likeNumFormat(item.likeCount)
        tv_count.setTextColor(if (item.isLike == 0) Color.parseColor("#C4C4C4") else Color.parseColor("#FF4F58"))

        iv_thumbs_up.setImageResource(if (item.isLike == 1) R.drawable.icon_like_s else R.drawable.icon_like_n)

        tv_biankanbianmai.visibility = if (item.skuList.isNotEmpty()) View.VISIBLE else View.GONE

        tv_deleted.visibility = if (item.deleted == 1 || item.state == 0) View.VISIBLE else View.GONE
        tv_deleted.text = if (item.state == 0) "此内容因违规已被删除" else "此内容已被删除"

        item_bg.setBackgroundColor(if (item.deleted == 1) Color.parseColor("#07000000") else Color.WHITE)

        val height = ScreenUtil.getScreenWidth(mContext) / 2 - UIUtil.dip2px(mContext, 39.0)
        val tempHeight: Int
        if (item.video.isNullOrEmpty()) {
            rv_time.visibility = View.GONE
            tempHeight = height
        } else {
            rv_time.visibility = View.VISIBLE
            time_tv.text = timeFormat(item.videoDuration)
            time_tv.typeface = TypefaceUtil.getTypeface(mContext)
            tempHeight = when (item.screenMode) {
                1 -> height - UIUtil.dip2px(mContext, 30.0)
                2 -> height
                3 -> height + UIUtil.dip2px(mContext, 60.0)
                else -> height
            }
        }

        val localLayoutParams = item_bg.layoutParams
        val deletedLayoutParams = tv_deleted.layoutParams

        deletedLayoutParams.height = tempHeight
        localLayoutParams.height = tempHeight

        helper.addOnClickListener(R.id.ll_thumbs_up)
    }

    /**
     * 传过来的秒格式话输出
     *
     * @param time
     * @return
     */
    private fun timeFormat(time: Int): String {
        val fen = time / 60
        val miao = time % 60
        return String.format("%02d:%02d", fen, miao)
    }
}