package com.gfeng.daydaycook.video

import android.animation.ValueAnimator
import android.content.Context
import android.content.Intent
import android.media.MediaPlayer
import android.os.Bundle
import android.os.Handler
import android.os.Message
import android.provider.MediaStore
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.view.MotionEvent
import android.view.SurfaceHolder
import android.view.View
import android.view.ViewConfiguration
import android.view.animation.LinearInterpolator
import android.widget.FrameLayout
import android.widget.RelativeLayout
import android.widget.Toast
import com.gfeng.daydaycook.R
import com.gfeng.daydaycook.activity.BaseActivity
import com.gfeng.daydaycook.comm.Comm
import com.gfeng.daydaycook.comm.Global
import com.gfeng.daydaycook.publish.PhotoActivity
import com.gfeng.daydaycook.publish.PublishActivity
import com.gfeng.daydaycook.publish.entity.ItemMediaBean
import com.gfeng.daydaycook.util.LogUtils
import com.gfeng.daydaycook.video.adapter.ThumbAdapter
import com.gfeng.daydaycook.video.entity.VideoEditInfo
import com.gfeng.daydaycook.video.util.*
import com.gfeng.daydaycook.video.widget.RangeSeekBar
import com.gfeng.daydaycook.widget.ToastDialog
import kotlinx.android.synthetic.main.activity_video_cut.*
import net.lucode.hackware.magicindicator.buildins.UIUtil
import java.io.File
import java.lang.ref.WeakReference

/**
 * Life Is Better With DayDayCook
 * author: HuangQiang
 * created on: 2019/3/8 16:07.
 * description: 视频剪切界面
 */
class VideoCutActivity : BaseActivity() {

    private val TAG = "VideoCutActivity"
    private var mExtractVideoInfoUtil: ExtractVideoInfoUtil? = null
    private var path: String = ""
    private var mMaxWidth: Float = 0f
    private var mScaledTouchSlop: Int = 0//触发移动事件的最短距离
    private var leftProgress: Long = 0
    private var rightProgress: Long = 0
    private var currentProgress: Long = 0

    private var duration: Long = 0
    private var scrollPos: Long = 0
    private var marginSpace: Float = 0f

    private val MIN_CUT_DURATION = 3 * 1000L// 最小剪辑时间3s
    private val MAX_CUT_DURATION = 3 * 60 * 1000L//视频最多剪切多长时间
    private val MAX_COUNT_RANGE = 8//seekBar的区域内一共有多少张图片

    private var averageMsPx: Float = 0f//每毫秒所占的px
    private var averagePxMs: Float = 0f//每px所占用的ms毫秒

    private var thumbAdapter: ThumbAdapter? = null
    private var mExtractFrameWorkThread: ExtractFrameWorkThread? = null
    private var OutPutFileDirPath: String? = null
    private var animator: ValueAnimator? = null

    private var hasStop: Boolean = false
    private var isSeeking: Boolean = false
    private var seekBar: RangeSeekBar? = null
    private var thumbnailsCount: Int = 0//有多少张缩略图
    private var lastThumbWidth: Int = 0//最后一张缩略图的宽度
    private var screenMode = 3
    private var rate: Float = 1f


    companion object {
        fun startActivity(context: Context, path: String) {
            val intent = Intent(context, VideoCutActivity::class.java)
            intent.putExtra("path", path)
            context.startActivity(intent)
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_video_cut)

        initData()
        initVideoThumb()//缩略图
        initView()
        initSurfaceView()
        mImmersionBar.titleBarMarginTop(titleLayout)
    }


    private fun initData() {
        path = intent.getStringExtra("path")
        if (!File(path).exists()) {
            Toast.makeText(this, "视频文件不存在", Toast.LENGTH_LONG).show()
            finish()
        }
        mExtractVideoInfoUtil = ExtractVideoInfoUtil(path)
        duration = java.lang.Long.valueOf(mExtractVideoInfoUtil?.videoLength ?: "0")
        marginSpace = resources.getDimension(R.dimen.margin_14) + resources.getDimension(R.dimen.margin_14)
        mMaxWidth = UIUtil.getScreenWidth(this) - 2 * marginSpace

        mScaledTouchSlop = ViewConfiguration.get(this).scaledTouchSlop
    }


    private fun initVideoThumb() {
        val extractW = mMaxWidth / MAX_COUNT_RANGE  //recyclerView每个item的宽度
        val extractH = UIUtil.dip2px(this, 50.0)
        leftProgress = 0

        //for video edit
        val startPosition: Long = 0
        val endPosition = duration
        val isOverMaxDuration: Boolean
        if (endPosition <= MAX_CUT_DURATION) {
            isOverMaxDuration = false
            rightProgress = endPosition
            thumbnailsCount = MAX_COUNT_RANGE

            averagePxMs = mMaxWidth * 1.0f / (rightProgress - leftProgress)
            averageMsPx = duration * 1.0f / mMaxWidth * 1.0f

        } else {
            isOverMaxDuration = true
            rightProgress = MAX_CUT_DURATION

            averageMsPx = MAX_CUT_DURATION * 1.0f / mMaxWidth * 1.0f
            averagePxMs = mMaxWidth * 1.0f / MAX_CUT_DURATION * 1.0f


            if (((endPosition * MAX_COUNT_RANGE) % MAX_CUT_DURATION) == 0L) {
                thumbnailsCount = ((endPosition * MAX_COUNT_RANGE) / MAX_CUT_DURATION).toInt()

            } else {
                thumbnailsCount = ((endPosition * MAX_COUNT_RANGE) / MAX_CUT_DURATION).toInt() + 1

                lastThumbWidth = (endPosition * averagePxMs - (thumbnailsCount - 1) * extractW).toInt()

            }
        }

        //init seekBar
        if (isOverMaxDuration) {
            seekBar = RangeSeekBar(this, 0L, MAX_CUT_DURATION)
            seekBar?.selectedMinValue = 0L
            seekBar?.selectedMaxValue = MAX_CUT_DURATION
        } else {
            seekBar = RangeSeekBar(this, 0L, endPosition)
            seekBar?.selectedMinValue = 0L
            seekBar?.selectedMaxValue = endPosition
        }
        seekBar?.setMin_cut_time(MIN_CUT_DURATION)//设置最小裁剪时间
        seekBar?.isNotifyWhileDragging = true
        seekBar?.setOnRangeSeekBarChangeListener(mOnRangeSeekBarChangeListener)
        ll_seekBar.addView(seekBar)

        OutPutFileDirPath = Comm.PATH_VIDEO_THUMB

        mExtractFrameWorkThread = ExtractFrameWorkThread(extractW.toInt(), extractH, mUIHandler, path, OutPutFileDirPath, startPosition, endPosition, thumbnailsCount)
        mExtractFrameWorkThread?.start()


    }

    private fun initView() {

        thumbRv.layoutManager = LinearLayoutManager(this, LinearLayoutManager.HORIZONTAL, false)
        thumbAdapter = ThumbAdapter(this,
                (mMaxWidth / MAX_COUNT_RANGE).toInt(), thumbnailsCount, lastThumbWidth)
        thumbRv.adapter = thumbAdapter
        thumbRv.addOnScrollListener(mOnScrollListener)
        backIv.setOnClickListener {
            onBackPressed()
        }
        nextTv.setOnClickListener {
            videoPause()
            VideoTrimmerUtil.trim(VideoCutActivity@ this, path, Comm.PATH_VIDEO_TRIM, leftProgress, rightProgress, trimListener)
        }
        setSelectedDuration()
    }

    private fun setSelectedDuration() {
        var selectedDuration = Math.floor((rightProgress / 1000f - leftProgress / 1000f).toDouble()).toLong()
        if (selectedDuration < 3)
            selectedDuration = 3
        else if (selectedDuration > 180)
            selectedDuration = 180
        selectedDurationTv.text = "已选时长 " + VideoTrimmerUtil.convertSecondsToTime(selectedDuration)
    }


    private var player: MediaPlayer? = null

    private var holder: SurfaceHolder? = null

    /**
     * SurfaceView初始化
     */
    private fun initSurfaceView() {

        surfaceView.setOnClickListener {
            if (!hasStop) {
                videoPause()
                hasStop = true
            } else {
                videoStart(currentProgress)
                hasStop = false
            }
        }
        holder = surfaceView.holder
        holder?.setKeepScreenOn(true)

        player = MediaPlayer()

        //设置OnPrepared监听
        player?.setOnPreparedListener {
            LogUtils.e(TAG, "OnPrepared")

            //设置MediaPlayer的OnSeekComplete监听
            it.setOnSeekCompleteListener { _ ->
                currentProgress = leftProgress
                LogUtils.e(TAG, "SeekComplete currentProgress=$currentProgress")
            }

            videoSeekAndStart(leftProgress)
        }
        player?.setOnVideoSizeChangedListener { mp, width, height ->
            changeVideoSize()
        }

        holder?.addCallback(object : SurfaceHolder.Callback {
            override fun surfaceChanged(holder: SurfaceHolder?, format: Int, width: Int, height: Int) {
                LogUtils.e(TAG, "surfaceChanged")
            }

            override fun surfaceDestroyed(holder: SurfaceHolder?) {
                LogUtils.e(TAG, "surfaceDestroyed")
                videoPause()
            }

            override fun surfaceCreated(holder: SurfaceHolder?) {
                LogUtils.e(TAG, "surfaceCreated")
                player?.setDisplay(holder)
                player?.reset()
                player?.setDataSource(path)
                player?.prepareAsync()

            }

        })
    }

    /**
     * surfaceView填充屏幕
     */
    private fun changeVideoSize() {
        var videoWidth = player?.videoWidth ?: 0
        var videoHeight = player?.videoHeight ?: 0

        val surfaceWidth = surfaceView.width
        val surfaceHeight = surfaceView.height


        //竖屏模式下按视频宽度计算放大倍数值
//        val max = Math.max(videoWidth.toFloat() / surfaceWidth.toFloat(), videoHeight.toFloat() / surfaceHeight.toFloat())

        //填充宽度
        val max = videoWidth.toFloat() / surfaceWidth.toFloat()


        //视频宽高分别/最大倍数值 计算出放大后的视频尺寸
        videoWidth = Math.ceil((videoWidth / max).toDouble()).toInt()
        videoHeight = Math.ceil((videoHeight / max).toDouble()).toInt()

        //无法直接设置视频尺寸，将计算出的视频尺寸设置到surfaceView 让视频自动填充。
        val params = RelativeLayout.LayoutParams(videoWidth, videoHeight)
        params.addRule(RelativeLayout.CENTER_VERTICAL, surfaceView.id)
        surfaceView.layoutParams = params

        rate = videoWidth * 1.0f / videoHeight * 1.0f


        screenMode = if (rate > 4.0f / 3.0f)
            1
        else if ((rate < (4.0f / 3.0f)) && (rate > (3.0f / 4.0f)))
            2
        else
            3

        LogUtils.e(TAG, "changeVideoSize screenMode=$screenMode")
    }


    /**
     * 视频进度指针图片动画启动
     */
    private fun positionViewAnim(progress: Long) {
        positionView.visibility = View.VISIBLE
        val params = positionView.layoutParams as FrameLayout.LayoutParams
        val start = (marginSpace + (progress - scrollPos) * averagePxMs).toInt()
        val end = (marginSpace + (rightProgress - scrollPos) * averagePxMs).toInt()
        LogUtils.e(TAG, "positionViewAnim progress=$progress rightProgress=$rightProgress")

        if (rightProgress - scrollPos < progress - scrollPos)
            return

        animator = ValueAnimator
                .ofInt(start, end)
                .setDuration(rightProgress - scrollPos - (progress - scrollPos))
        animator?.interpolator = LinearInterpolator()
        animator?.addUpdateListener { animation ->
            params.leftMargin = animation.animatedValue as Int
            positionView.layoutParams = params
        }
        animator?.start()
    }

    /**
     * 视频进度指针图片动画复原
     */
    private fun positionViewReset() {
        LogUtils.e(TAG, "positionViewReset")

        handler.removeCallbacks(run)
        positionView.clearAnimation()
        if (animator?.isRunning == true) {
            animator?.cancel()
        }
    }

    private fun videoStart(progress: Long) {

        LogUtils.e(TAG, "videoStart progress=$progress")

        player?.start()
        positionViewReset()
        positionViewAnim(progress)
        handler.post(run)
        playIv.visibility = View.GONE
        hasStop = false
    }

    private fun videoSeekAndStart(progress: Long) {

        LogUtils.e(TAG, "videoSeekAndStart progress=$progress")
        leftProgress = progress
        player?.seekTo(progress.toInt())
        videoStart(progress)
    }


    private fun videoPause() {
        LogUtils.e(TAG, "videoPause currentProgress=$currentProgress")

        if (player?.isPlaying == true)
            player?.pause()
        positionViewReset()

        playIv.visibility = View.VISIBLE
        currentProgress = player?.currentPosition?.toLong() ?: 0

        hasStop = true

    }

    private val mOnRangeSeekBarChangeListener = RangeSeekBar.OnRangeSeekBarChangeListener { _, minValue, maxValue, action, _, pressedThumb ->
        LogUtils.e(TAG, "-----minValue----->>>>>>$minValue")
        LogUtils.e(TAG, "-----maxValue----->>>>>>$maxValue")
        leftProgress = minValue + scrollPos
        rightProgress = maxValue + scrollPos
        LogUtils.e(TAG, "-----leftProgress----->>>>>>$leftProgress")
        LogUtils.e(TAG, "-----rightProgress----->>>>>>$rightProgress")
        when (action) {
            MotionEvent.ACTION_DOWN -> {
                LogUtils.e(TAG, "-----ACTION_DOWN---->>>>>>")
                isSeeking = false
                videoPause()
                positionView.visibility = View.GONE
            }
            MotionEvent.ACTION_MOVE -> {
                LogUtils.e(TAG, "-----ACTION_MOVE---->>>>>>")
                isSeeking = true
                player?.seekTo((if (pressedThumb === RangeSeekBar.Thumb.MIN)
                    leftProgress
                else
                    rightProgress).toInt())

                setSelectedDuration()
            }
            MotionEvent.ACTION_UP -> {
                LogUtils.e(TAG, "-----ACTION_UP--leftProgress--->>>>>>$leftProgress")
                isSeeking = false
                //从minValue开始播
                player?.seekTo(leftProgress.toInt())
            }
            else -> {

            }
        }
    }

    private val handler = Handler()
    private var run = object : Runnable {

        override fun run() {
            val currentPosition = player?.currentPosition?.toLong() ?: 0
            LogUtils.e(TAG, "Runnable currentPosition=$currentPosition currentPosition=$currentPosition rightProgress=$rightProgress leftProgress=$leftProgress")

            if (currentPosition >= rightProgress || (currentPosition == currentProgress && currentPosition != leftProgress)) {
                videoSeekAndStart(leftProgress)
                currentProgress = leftProgress

            } else {
                handler.postDelayed(this, 1000)
                currentProgress = currentPosition
            }
        }
    }

    private var isOverScaledTouchSlop: Boolean = false
    private var lastScrollX = 0

    private val mOnScrollListener = object : RecyclerView.OnScrollListener() {
        override fun onScrollStateChanged(recyclerView: RecyclerView, newState: Int) {
            super.onScrollStateChanged(recyclerView, newState)
//            LogUtils.e(TAG, "-------newState:>>>>>$newState")
            if (newState == RecyclerView.SCROLL_STATE_IDLE) {
                isSeeking = false
                //                videoStart();
            } else {
                isSeeking = true
                if (isOverScaledTouchSlop) {
                    videoPause()
                }
            }
        }

        override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
            super.onScrolled(recyclerView, dx, dy)
//            LogUtils.e(TAG, "-------dx:>>>>>$dx")

            isSeeking = false
            val scrollX = getScrollXDistance()
            //达不到滑动的距离
            if (Math.abs(lastScrollX - scrollX) < mScaledTouchSlop) {
                isOverScaledTouchSlop = false
                return
            }
//            LogUtils.e(TAG, "-------scrollX:>>>>>$scrollX")

            isOverScaledTouchSlop = true

            videoPause()

            positionView.visibility = View.GONE
            isSeeking = true
            scrollPos = (averageMsPx * scrollX).toLong()
            leftProgress = seekBar!!.selectedMinValue + scrollPos
            rightProgress = seekBar!!.selectedMaxValue + scrollPos
//            LogUtils.e(TAG, "scrollPos=$scrollPos rightProgress=$rightProgress leftProgress=$leftProgress")
            player?.seekTo(leftProgress.toInt())

            lastScrollX = scrollX
        }
    }

    /**
     * 水平滑动了多少px
     *
     * @return int px
     */
    private fun getScrollXDistance(): Int {
        val layoutManager = thumbRv.layoutManager as LinearLayoutManager
        val position = layoutManager.findFirstVisibleItemPosition()
        val firstVisibleChildView = layoutManager.findViewByPosition(position)
        val itemWidth = firstVisibleChildView?.width ?: 0
        return position * itemWidth - (firstVisibleChildView?.left ?: 0)
    }

    private val trimListener = object : VideoTrimListener {

        override fun onStartTrim() {
            LogUtils.e(TAG, "视频截取开始")
            showProgressDialog(false, "处理中...")

        }

        override fun onSuccess(trimResult: String?) {
            LogUtils.e(TAG, "视频截取结束 url=$trimResult")

            VideoCompressor.compress(this@VideoCutActivity, trimResult, Comm.PATH_VIDEO_COMPRESS, rate, object : VideoHandleListener {
                override fun onSuccess(compressResult: String?) {
                    LogUtils.e(TAG, "视频压缩成功url=$compressResult")

                    VideoGifUtil.gif(this@VideoCutActivity, trimResult, Comm.PATH_VIDEO_GIF, rate, object : VideoHandleListener {
                        override fun onSuccess(gifResult: String?) {
                            LogUtils.e(TAG, "视频GIF成功")
                            goToPublish(compressResult, gifResult)
                        }

                        override fun onFailure() {
                            LogUtils.e(TAG, "视频GIF失败")

                            hideProgressDialog()
                            ToastDialog.showToastDialog(this@VideoCutActivity, R.drawable.icon_toast_message, "视频生成gif失败", 2000)

                        }
                    })

                }

                override fun onFailure() {
                    hideProgressDialog()
                    ToastDialog.showToastDialog(this@VideoCutActivity, R.drawable.icon_toast_message, "视频压缩失败", 2000)
                }
            })


        }

        override fun onFailure() {
            hideProgressDialog()
            ToastDialog.showToastDialog(this@VideoCutActivity, R.drawable.icon_toast_message, "视频剪切失败", 2000)
        }


    }

    fun goToPublish(compressResult: String?, gifResult: String?) {
        hideProgressDialog()
        val videoList = ArrayList<ItemMediaBean>()
        val itemMediaBean = ItemMediaBean()
        itemMediaBean.videoUrl = compressResult
        itemMediaBean.url = gifResult
        itemMediaBean.videoDuration = rightProgress - leftProgress
        itemMediaBean.mediaType = MediaStore.Files.FileColumns.MEDIA_TYPE_VIDEO
        itemMediaBean.screenMode = screenMode
        videoList.add(itemMediaBean)
        val intent = Intent()
        intent.setClass(this@VideoCutActivity, PublishActivity::class.java)
        intent.putExtra(PublishActivity.MEDIA_TYPE_KEY, MediaStore.Files.FileColumns.MEDIA_TYPE_VIDEO)
        intent.putExtra(PhotoActivity.INTENT_KEY, videoList)
        startActivity(intent)
        Global.mAppMgr.refreshActivityData(Comm.photoactivity_refresh_type, PhotoActivity.finish_type, null)

        finish()
    }


    override fun onDestroy() {
        super.onDestroy()

        if (player?.isPlaying == true) {
            player?.stop()
        }
        player?.release()

        animator?.cancel()

        thumbRv.removeOnScrollListener(mOnScrollListener)

        mExtractVideoInfoUtil?.release()
        mExtractFrameWorkThread?.stopExtract()
        mUIHandler.removeCallbacksAndMessages(null)
        handler.removeCallbacksAndMessages(null)

//        if (!TextUtils.isEmpty(OutPutFileDirPath)) {
//            PictureUtils.deleteFile(File(OutPutFileDirPath))
//        }
    }


    private val mUIHandler = MainHandler(this)

    private class MainHandler internal constructor(activity: VideoCutActivity) : Handler() {
        private val mActivity: WeakReference<VideoCutActivity> = WeakReference(activity)

        override fun handleMessage(msg: Message) {
            val activity = mActivity.get()
            if (activity != null) {
                if (msg.what == ExtractFrameWorkThread.MSG_SAVE_SUCCESS) {
                    if (activity.thumbAdapter != null) {
                        val info = msg.obj as VideoEditInfo
                        activity.thumbAdapter!!.addItemVideoInfo(info)
                    }
                }
            }
        }
    }

}